<?php

function sortBySubkey(&$array, $subkey, $sortType = SORT_ASC)
{
    $all_keys = array_keys($array);

    array_multisort(array_column($array, $subkey), $sortType, SORT_NUMERIC, $array, $all_keys);
    $array = array_combine($all_keys, $array);
}

if (!function_exists('array_column')) {
    function array_column($input = null, $columnKey = null, $indexKey = null)
    {
        // Using func_get_args() in order to check for proper number of
        // parameters and trigger errors exactly as the built-in array_column()
        // does in PHP 5.5.
        $argc = func_num_args();
        $params = func_get_args();
        if ($argc < 2) {
            trigger_error("array_column() expects at least 2 parameters, {$argc} given", E_USER_WARNING);
            return null;
        }
        if (!is_array($params[0])) {
            trigger_error('array_column() expects parameter 1 to be array, ' . gettype($params[0]) . ' given', E_USER_WARNING);
            return null;
        }
        if (!is_int($params[1]) && !is_float($params[1]) && !is_string($params[1]) && $params[1] !== null && !(is_object($params[1]) && method_exists($params[1], '__toString'))) {
            trigger_error('array_column(): The column key should be either a string or an integer', E_USER_WARNING);
            return false;
        }
        if (isset($params[2]) && !is_int($params[2]) && !is_float($params[2]) && !is_string($params[2]) && !(is_object($params[2]) && method_exists($params[2], '__toString'))) {
            trigger_error('array_column(): The index key should be either a string or an integer', E_USER_WARNING);
            return false;
        }
        $paramsInput = $params[0];
        $paramsColumnKey = ($params[1] !== null) ? (string)$params[1] : null;
        $paramsIndexKey = null;
        if (isset($params[2])) {
            if (is_float($params[2]) || is_int($params[2])) {
                $paramsIndexKey = (int)$params[2];
            } else {
                $paramsIndexKey = (string)$params[2];
            }
        }
        $resultArray = [];
        foreach ($paramsInput as $row) {
            $key = $value = null;
            $keySet = $valueSet = false;
            if ($paramsIndexKey !== null && array_key_exists($paramsIndexKey, $row)) {
                $keySet = true;
                $key = (string)$row[$paramsIndexKey];
            }
            if ($paramsColumnKey === null) {
                $valueSet = true;
                $value = $row;
            } elseif (is_array($row) && array_key_exists($paramsColumnKey, $row)) {
                $valueSet = true;
                $value = $row[$paramsColumnKey];
            }
            if ($valueSet) {
                if ($keySet) {
                    $resultArray[$key] = $value;
                } else {
                    $resultArray[] = $value;
                }
            }
        }
        return $resultArray;
    }
}

/**
 * Class Model ControllerExtensionForDealerImport
 * @property ModelToolForDealerImport model_extension_for_dealer_import $model_extension_for_dealer_import
 */
class ControllerExtensionForDealerImport extends Controller
{
    private $error = [];
    private $ssl = 'SSL';

    private $maxProcessesPerJobs;
    private $shortUpdateNumRows;
    private $fullUpdateNumRows;

    private $loggerEnabled;

    const IMAGE_PATH = 'catalog/4dealer/';


    protected $dealer_no_id;
    protected $dealer_name_id;

    public function __construct($registry)
    {
        parent::__construct($registry);
        ini_set('default_socket_timeout', 600);

        $this->initDefaults();

        $this->load->model('extension/for_dealer_import');

        if (file_exists(__DIR__ . "/oc_cli_mod.zip")) {
            $this->install();
        }
        $attributes_data = $this->model_extension_for_dealer_import->getForDealerAttributes();

        if ($attributes_data) {
            foreach ($attributes_data as $attr_data) {
                switch ($attr_data['name']) {
                    case 'Поставщик':
                        $this->dealer_name_id = $attr_data['attribute_id'];
                        break;
                    case 'Номер товара в ИС поставщика':
                        $this->dealer_no_id = $attr_data['attribute_id'];
                        break;
                }
            }
        }
    }

    protected function initDefaults()
    {
        $this->ssl = true;

        $prefix = 'for_dealer_import_settings_';

        $this->maxProcessesPerJobs = (int) $this->config->get($prefix . 'calls_per_iteration');
        $this->maxProcessesPerJobs = $this->maxProcessesPerJobs ? $this->maxProcessesPerJobs : 5;

        $this->shortUpdateNumRows = (int) $this->config->get($prefix . 'short_step');
        $this->shortUpdateNumRows = $this->shortUpdateNumRows ? $this->shortUpdateNumRows : 150;

        $this->fullUpdateNumRows = (int) $this->config->get($prefix . 'full_step');
        $this->fullUpdateNumRows = $this->fullUpdateNumRows ? $this->fullUpdateNumRows : 30;

        $this->loggerEnabled = $this->config->get($prefix . 'logger') === 'on';
    }

    public function install()
    {
        $this->model_extension_for_dealer_import->createSchemas();
        $zip = new ZipArchive;

        if ($zip->open(__DIR__ . "/oc_cli_mod.zip") === true) {
            if ($zip->extractTo(DIR_APPLICATION . '../')) {
                $zip->close();
                unlink(__DIR__ . "/oc_cli_mod.zip");
            }
        }

        //$zip=zip_open(__DIR__."/oc_cli_mod.zip");
        //rename(DIR_SYSTEM."config/oc_cli_root.txt",DIR_APPLICATION."../oc_cli.php");
        //rename(__DIR__."/oc_cli",dirname(__DIR__)."/oc_cli");
    }


    public function index()
    {
        $this->load->language('extension/for_dealer_import');
        $this->document->setTitle($this->language->get('heading_title'));
        $this->getForm();
    }


    protected function return_bytes($val)
    {
        $val = trim($val);

        switch (strtolower(substr($val, -1))) {
            case 'm':
                $val = (int)substr($val, 0, -1) * 1048576;
                break;
            case 'k':
                $val = (int)substr($val, 0, -1) * 1024;
                break;
            case 'g':
                $val = (int)substr($val, 0, -1) * 1073741824;
                break;
            case 'b':
                switch (strtolower(substr($val, -2, 1))) {
                    case 'm':
                        $val = (int)substr($val, 0, -2) * 1048576;
                        break;
                    case 'k':
                        $val = (int)substr($val, 0, -2) * 1024;
                        break;
                    case 'g':
                        $val = (int)substr($val, 0, -2) * 1073741824;
                        break;
                    default :
                        break;
                }
                break;
            default:
                break;
        }
        return $val;
    }


    public function settings()
    {
        $this->load->language('extension/for_dealer_import');
        $this->document->setTitle($this->language->get('heading_title'));
        $this->load->model('extension/for_dealer_import');
        if (($this->request->server['REQUEST_METHOD'] == 'POST') && ($this->validateSettingsForm())) {
            $this->load->model('setting/setting');
            $this->model_setting_setting->editSetting('for_dealer_import', $this->request->post);
            $this->session->data['success'] = $this->language->get('text_success_settings');
            $this->response->redirect($this->url->link('extension/for_dealer_import', 'user_token=' . $this->session->data['user_token'], $this->ssl));
        }
        $this->getForm();
    }


    protected function getForm()
    {
        $data = [];

        $data['query'] = $_GET;

        $data['heading_title'] = $this->language->get('heading_title');

        $data['tab_export'] = $this->language->get('tab_export');
        $data['tab_import'] = $this->language->get('tab_import');
        $data['tab_settings'] = $this->language->get('tab_settings');
        $data['tab_dealers_list'] = $this->language->get('tab_dealers_list');
        $data['tab_dealers_cat'] = $this->language->get('tab_dealers_cat');
        $data['tab_dealers_settings'] = $this->language->get('tab_dealers_settings');

        $data['button_export'] = $this->language->get('button_export');
        $data['button_import'] = $this->language->get('button_import');
        $data['button_settings'] = $this->language->get('button_settings');
        $data['button_export_id'] = $this->language->get('button_export_id');
        $data['button_export_page'] = $this->language->get('button_export_page');


        $data['error_select_file'] = $this->language->get('error_select_file');
        $data['error_post_max_size'] = str_replace('%1', ini_get('post_max_size'), $this->language->get('error_post_max_size'));
        $data['error_upload_max_filesize'] = str_replace('%1', ini_get('upload_max_filesize'), $this->language->get('error_upload_max_filesize'));
        $data['error_id_no_data'] = $this->language->get('error_id_no_data');
        $data['error_page_no_data'] = $this->language->get('error_page_no_data');
        $data['error_param_not_number'] = $this->language->get('error_param_not_number');
        $data['error_notifications'] = $this->language->get('error_notifications');
        $data['error_no_news'] = $this->language->get('error_no_news');
        $data['error_batch_number'] = $this->language->get('error_batch_number');
        $data['error_min_item_id'] = $this->language->get('error_min_item_id');

        if (!empty($this->session->data['for_dealer_import_error']['errstr'])) {
            $this->error['warning'] = $this->session->data['for_dealer_import_error']['errstr'];
        }

        if (isset($this->error['warning'])) {
            $data['error_warning'] = $this->error['warning'];

            if (!empty($this->session->data['for_dealer_import_nochange'])) {
                $data['error_warning'] .= "<br />\n" . $this->language->get('text_nochange');
            }

        } else {
            $data['error_warning'] = '';
        }

        unset($this->session->data['for_dealer_import_error']);
        unset($this->session->data['for_dealer_import_nochange']);

        if (isset($this->session->data['success'])) {
            $data['success'] = $this->session->data['success'];

            unset($this->session->data['success']);
        } else {
            $data['success'] = '';
        }

        $data['breadcrumbs'] = [];
        $data['breadcrumbs'][] = [
            'text' => $this->language->get('text_home'),
            'href' => $this->url->link('common/dashboard', 'user_token=' . $this->session->data['user_token'], $this->ssl)
        ];

        $data['breadcrumbs'][] = array(
            'text' => $this->language->get('heading_title'),
            'href' => $this->url->link('extension/for_dealer_import', 'user_token=' . $this->session->data['user_token'], $this->ssl)
        );

        $data['back'] = $this->url->link('common/dashboard', 'user_token=' . $this->session->data['user_token'], $this->ssl);
        $data['button_back'] = $this->language->get('button_back');
        $data['import'] = $this->url->link('extension/for_dealer_import/upload', 'user_token=' . $this->session->data['user_token'], $this->ssl);
        $data['export'] = $this->url->link('extension/for_dealer_import/download', 'user_token=' . $this->session->data['user_token'], $this->ssl);
        $data['settings'] = $this->url->link('extension/for_dealer_import/settings', 'user_token=' . $this->session->data['user_token'], $this->ssl);
        $data['post_max_size'] = $this->return_bytes(ini_get('post_max_size'));
        $data['upload_max_filesize'] = $this->return_bytes(ini_get('upload_max_filesize'));


        if (isset($this->request->post['for_dealer_import_settings_login'])) {
            $data['settings_login'] = $this->request->post['for_dealer_import_settings_login'];
        } else $data['settings_login'] = $this->config->get('for_dealer_import_settings_login');

        if (isset($this->request->post['for_dealer_import_settings_token'])) {
            $data['settings_token'] = $this->request->post['for_dealer_import_settings_token'];
        } else $data['settings_token'] = $this->config->get('for_dealer_import_settings_token');

        if (isset($this->request->post['for_dealer_import_settings_hide_dup'])) {
            $data['hide_dup'] = $this->request->post['hide_dup'];
        } else $data['hide_dup'] = $this->config->get('for_dealer_import_settings_hide_dup');

        if (isset($this->request->post['for_dealer_import_settings_hide_dup_cond'])) {
            $data['hide_dup_cond'] = $this->request->post['hide_dup_cond'];
        } else $data['hide_dup_cond'] = $this->config->get('for_dealer_import_settings_hide_dup_cond');
        
        if (isset($this->request->post['for_dealer_import_settings_avail_cond'])) {
            $data['avail_cond'] = $this->request->post['avail_cond'];
        } else $data['avail_cond'] = $this->config->get('for_dealer_import_settings_avail_cond');

        if (isset($this->request->post['for_dealer_import_settings_exclude_str'])) {
            $data['exclude_str'] = $this->request->post['exclude_str'];
        } else $data['exclude_str'] = $this->config->get('for_dealer_import_settings_exclude_str');

        if (isset($this->request->post['for_dealer_import_settings_calls_per_iteration'])) {
            $data['calls_per_iteration'] = $this->request->post['calls_per_iteration'];
        } else $data['calls_per_iteration'] = $this->maxProcessesPerJobs;

        if (isset($this->request->post['for_dealer_import_settings_short_step'])) {
            $data['short_step'] = $this->request->post['short_step'];
        } else $data['short_step'] = $this->shortUpdateNumRows;

        if (isset($this->request->post['for_dealer_import_settings_full_step'])) {
            $data['full_step'] = $this->request->post['full_step'];
        } else $data['full_step'] = $this->fullUpdateNumRows;

        if (isset($this->request->post['for_dealer_import_settings_logger'])) {
            $data['logger'] = $this->request->post['logger'] === 'on';
        } else $data['logger'] = $this->loggerEnabled;

        $data['user_token'] = $this->session->data['user_token'];

        $lang_code = $this->config->get('config_language');
        $price_list = $this->prepareDealersList();
        $data['dealers_list'] = $price_list;

        $data['current_tab'] = isset($this->request->get['tab']) ? $this->request->get['tab'] : 'dealers_list';

        $selected_dealer = isset($_REQUEST['dealer_id']) ? (int)$_REQUEST['dealer_id'] : false;

        $categories_data = $this->prepareCatMap($selected_dealer);

        $data['processJobsList'] = $this->getProcessJobsList();

        $data['dealer_categories'] = '';

        if ($selected_dealer) {
            $this->showArrayTree($categories_data['dealer_categories'], $categories_data['mappings'], $categories_data['local_categories'], $lang_code, $data['dealer_categories']);

            $data['dealers_list']['selected_id'] = $categories_data['selected_id'];
            $data['dealer_detail'] = $this->prepareDealerDetail($lang_code);

            if (isset($_REQUEST['section_id'])) {
                $data['options_merge'] = $this->model_extension_for_dealer_import
                    ->getExistOptionsMergeBySectionId((int)$_REQUEST['dealer_id'], (int)$_REQUEST['section_id']);

                foreach ($data['options_merge'] as &$option) {
                    $option['count_replaces'] = isset($option['replaces']) && $option['replaces'] ? count(json_decode($option['replaces'], true)) : 0;
                }
            }

        }

        $data['selectedRoundPrice'] = ['', '', -1 => '', -2 => '', -3 => ''];
        $data['selectedRoundRule'] = ['', ''];

        if (isset($data['dealer_detail'])) {
            $data['selectedRoundPrice'][ $data['dealer_detail']['dealer']['round_price'] ] = 'selected';
            $data['selectedRoundRule'][ $data['dealer_detail']['dealer']['round_rule'] ] = 'selected';
        }

        $data['local_categories'] = '';

        $data['isActualVersion'] = $this->model_extension_for_dealer_import->isActualVersion();
        $data['actualVersion'] = $this->model_extension_for_dealer_import->actualPluginVersion();

        $data['countSectionMergeOptions'] = json_encode(isset($data['options_merge_count']) ? $data['options_merge_count'] : []);

        $this->showObjectTree($categories_data['local_categories_tree'], $lang_code, $data['local_categories']);

        $this->document->addStyle('view/stylesheet/for_dealer_import.css?v=' . $data['actualVersion']);
        $this->document->addStyle('view/stylesheet/for_dealer_bootstrap.css');

        $this->document->addStyle('view/stylesheet/for_dealer_jquery.notify.css');
        $this->document->addStyle('view/stylesheet/for_dealer_jquery-ui.min.css');

        $this->document->addScript('view/javascript/oc-for_dealer_admin.js?v=' . $data['actualVersion']);
        $this->document->addScript('view/javascript/oc-for_dealer_bootstrap.bundle.min.js');
        $this->document->addScript('view/javascript/oc-for_dealer_jquery.notify.min.js');
        $this->document->addScript('view/javascript/oc-for_dealer_jquery.toast.js');
        $this->document->addScript('view/javascript/oc-for_dealer_jquery-ui.min.js');
        $this->document->addScript('view/javascript/oc-for_dealer_jquery.query-object.js');


        $data['header'] = $this->load->controller('common/header');
        $data['column_left'] = $this->load->controller('common/column_left');
        $data['footer'] = $this->load->controller('common/footer');

        $this->response->setOutput($this->load->view('extension/for_dealer_import', $data));

    }


    protected function validateSettingsForm()
    {
        if (!$this->user->hasPermission('access', 'extension/for_dealer_import')) {
            $this->error['warning'] = $this->language->get('error_permission');
            return false;
        }

        return true;
    }


    public function prepareDealersList()
    {
        $prices = $this->model_extension_for_dealer_import->getLocalPriceList();
        if (!$prices) {
            $prices = $this->model_extension_for_dealer_import->getRemotePriceList();
        }
        return ['prices' => ($prices) ? $prices->rows : false, 'jobs' => $this->model_extension_for_dealer_import->getJobsQueue()];
    }

    public function getDealerCategories()
    {
        $dealer_id = isset($_REQUEST['id']) ? (int)$_REQUEST['id'] : false;
        if (!$dealer_id) {
            return false;
        }
        $this->response->setOutput(json_encode($this->model_extension_for_dealer_import->getLocalDealerCategories($dealer_id)));

    }

    public function prepareCatMap($dealer_id = false)
    {
        //$exist_seo_url_table = $this->use_table_seo_url;
        $product_categories = $this->model_extension_for_dealer_import->getCategories();
        $data = ['prices' => $this->model_extension_for_dealer_import->getRemotePriceList(), 'local_categories' => $product_categories, 'local_categories_tree' => $this->buildCategoryTree($product_categories), 'selected_id' => 0,];
        if ($dealer_id) {
            $data['selected_id'] = $dealer_id;
            $data['dealer_categories'] = $this->model_extension_for_dealer_import->getLocalDealerCategories($data['selected_id']);
            $flat_categories_list = [];
            $this->buildFlatCategoriesList($data['dealer_categories'], $flat_categories_list);
            $data['dealer_categories_flat'] = $flat_categories_list;
            $data['mappings'] = $this->model_extension_for_dealer_import->getCategoriesMappingByPriceId($data['selected_id']);
        }
        return $data;
    }

    protected function buildFlatCategoriesList($categories, &$categories_list)
    {
        if (isset($categories) && is_array($categories)) {
            foreach ($categories as $category) {
                $categories_list[$category['id']] = $category;
                if (isset($category['children'])) {
                    unset($categories_list[$category['id']]['children']);
                    $this->buildFlatCategoriesList($category['children'], $categories_list);
                }
            }
            return $categories_list;
        }

        return null;
    }

    public function buildCategoryTree(&$elements, $parentId = 0, $parents_info = false)
    {
        $branch = [];
        foreach ($elements as $key => $element) {

            if ($element['parent_id'] != $parentId) {
                continue;
            }

            $children = $this->buildCategoryTree($elements, $element['category_id']);
            if (!empty($children)) {
                $element['children'] = $children;
            }
            unset($children);
            if ($parents_info) {
                $element['parents_array'][] = $parentId;
            }
            $branch[$element['category_id']] = $element;
        }
        unset($key, $element);
        return $branch;
    }


    protected function showArrayTree($tree, $mappings, $local_categories, $language_code, &$result)
    {
        $result .= "<ul>\n";
        foreach ($tree as $branch) {
            $add = '';
            $class = '';
            if (isset($mappings[$branch['id']])) {
                if ($local_category = $this->model_extension_for_dealer_import->searchCategoryById($local_categories, $mappings[$branch['id']])) {
                    $add = "<span>({$local_category['name'][$language_code]})</span><span class='clear_merge' data-owner-id='{$branch['id']}'></span>";
                    $class = 'class="merged"';
                }
            }
            $result .= "<li class='" . ($class ? 'merged-li' : '') . "'><span data-filter-item='' $class id=\"dealer-category-{$branch['id']}\" data-dealer-id=\"{$branch['id']}\">{$branch['name']}</span>\n<span>$add</span>";
            if (isset($branch['children'])) {
                $this->showArrayTree($branch['children'], $mappings, $local_categories, $language_code, $result);
            }
            $result .= "</li>\n";
        }
        $result .= "</ul>\n";
    }


    function showObjectTree($tree, $lang_code, &$result)
    {
        $result .= "<ul>\n";
        foreach ($tree as $branch) {
            $result .= "<li><span data-filter-item='' id='site-term-{$branch['category_id']}' data-term-id='{$branch['category_id']}'>{$branch['name'][$lang_code]}</span>";
            if (isset($branch['children'])) {
                $this->showObjectTree($branch['children'], $lang_code, $result);
            }
            $result .= "</li>\n";
        }
        $result .= "</ul>\n";
    }

    function importDealerCategoriesTree()
    {
        $result = ['res' => 0];
        if (isset($_REQUEST['dealer_id'])) {
            $res = $this->model_extension_for_dealer_import->dealerCategoriesTreeToTerm($this->model_extension_for_dealer_import->getLocalDealerCategories((int)$_REQUEST['dealer_id']), (int)$_REQUEST['dealer_id'], $this->model_extension_for_dealer_import->getDefaultLanguageId(), 0, [$this->config->get('config_store_id')]);
            if ($res) {
                $result['res'] = 1;
            }
        }
        $this->response->setOutput(json_encode($result));
    }


    function getProcessJobsList()
    {
        $page = isset($_REQUEST['jobs-list-page']) ? $_REQUEST['jobs-list-page'] : 1;
        return $this->model_extension_for_dealer_import->getProcessJobsList($page);
    }

    function updateJobById()
    {
        $job_id = $_REQUEST['job_id'];
        $data = $_REQUEST['job_data'];

        $this->response->setOutput(json_encode($this->model_extension_for_dealer_import->updateJobById($job_id, $data)));
    }

    function deleteJobById()
    {
        $job_id = isset($_REQUEST['job_id']) ? $_REQUEST['job_id'] : null;

        $this->response->setOutput(json_encode($this->model_extension_for_dealer_import->deleteJobById($job_id)));
    }

    function deleteFinishedJobs()
    {
        $this->response->setOutput(json_encode($this->model_extension_for_dealer_import->deleteFinishedJobs()));
    }


    function mergeCategories()
    {
        $this->response->setOutput(json_encode($this->model_extension_for_dealer_import->mergeCategories()));
    }

    function clearMergedCategories()
    {
        $this->response->setOutput(json_encode($this->model_extension_for_dealer_import->clearMergedCategories()));
    }


    function updateDealerParams()
    {
        $this->response->setOutput(json_encode($this->model_extension_for_dealer_import->updateDealerParams()));
    }

    function deletePriceMarkup()
    {
        $this->response->setOutput(json_encode($this->model_extension_for_dealer_import->deletePriceMarkup()));
    }

    function addPriceMarkup()
    {
        $this->response->setOutput(json_encode($this->model_extension_for_dealer_import->addPriceMarkup()));
    }

    function updatePriceMarkup()
    {
        $this->response->setOutput(json_encode($this->model_extension_for_dealer_import->updatePriceMarkup()));
    }

    function updatePriceMarkupAutoLoadOptions()
    {
        $this->response->setOutput(json_encode($this->model_extension_for_dealer_import->updatePriceMarkupAutoLoadOptions()));
    }


    function addOptionsMerge()
    {
        $this->response->setOutput(json_encode($this->model_extension_for_dealer_import->addOptionsMerge()));
    }

    function updateOptionsMerge()
    {
        $this->response->setOutput(json_encode($this->model_extension_for_dealer_import->updateOptionsMerge()));
    }

    function multipleCreateOptions()
    {
        $this->response->setOutput(json_encode($this->model_extension_for_dealer_import->multipleCreateOptions((int)$_REQUEST['price_id'], (int)$_REQUEST['section_id'])));
    }

    function getSectionOptionsList()
    {
        $price_id = $_REQUEST['price_id'];
        $section_id = $_REQUEST['section_id'];

        $dealerCats = $this->model_extension_for_dealer_import->getDealerCategoriesByLocalCatIds($price_id, [$section_id]);

        $options = (array) $this->model_extension_for_dealer_import->get_list_options_merge($dealerCats,false);

        $optionsList = array_column($options['Result'], 'option_name');
        $optionsList = array_unique(array_filter(array_unique($optionsList)));

        $this->response->setOutput(json_encode(array_values($optionsList), JSON_UNESCAPED_UNICODE));
    }

    function saveMergeReplaces()
    {
        $this->response->setOutput(json_encode($this->model_extension_for_dealer_import->saveMergeReplaces()));
    }

    function deleteOptionsMerge()
    {
        $this->response->setOutput(json_encode($this->model_extension_for_dealer_import->deleteOptionsMerge()));
    }

    /**
     * Удаление привязок к несуществующим категориям
     */
    function deleteEmptyBindings() {
        $this->response->setOutput(json_encode($this->model_extension_for_dealer_import->deleteEmptyBindings()));
    }


    function addJob()
    {
        $cmd_list = isset($_REQUEST['cmd_list']) ? $_REQUEST['cmd_list'] : false;
        if ($cmd_list) {
            $this->response->setOutput(json_encode($this->model_extension_for_dealer_import->addJob($cmd_list, null, true)));
        } else {
            $this->response->setOutput(json_encode(['res' => 0]));
        }
    }

    function getJobsStatusByArrayId()
    {
        $this->response->setOutput(json_encode($this->model_extension_for_dealer_import->getJobsStatusByArrayId()));
    }

    function cancelCommand()
    {
        $id = isset($_REQUEST['cmd_id']) ? (int)$_REQUEST['cmd_id'] : false;
        if (!$id) {
            $this->response->setOutput(json_encode(['res' => 0]));
        } else {
            $this->response->setOutput(json_encode($this->model_extension_for_dealer_import->cancelCommand($id, true)));
        }
    }

    function updateScheduleDate()
    {
        $this->response->setOutput(json_encode($this->model_extension_for_dealer_import->updateScheduleDate()));
    }

    function updateScheduleType()
    {
        $this->response->setOutput(json_encode($this->model_extension_for_dealer_import->updateScheduleType()));
    }


    public function prepareDealerDetail($language_code)
    {
        $data['selected_id'] = 0;

        if (isset($_REQUEST['dealer_id'])) {

            $data['selected_id'] = (int)$_REQUEST['dealer_id'];

            $local_categories = $this->model_extension_for_dealer_import->getLocalDealerCategories($data['selected_id']);

            $data['dealer'] = $this->model_extension_for_dealer_import->getPriceInfoById($data['selected_id']);
            $data['price_markups'] = $this->model_extension_for_dealer_import->getPriceMarkupsByDealerId($data['selected_id']);

            $data['local_merged_categories'] = $this->model_extension_for_dealer_import->getCategoriesTermsList($data['selected_id']);

            $lev = 1;
            $options = "<option value='0'>---все---</option>\n";

            if (is_array($data['price_markups'])) {
                foreach ($data['price_markups'] as $price_markup) {
                    $data['price_markups_by_categories'][$price_markup['category_id']] = $this->showArrayTreeSelect($price_markup['category_id'], $local_categories, $lev, $options);
                }
                unset($price_markup);
            }
            //unset($data['price_markups']);
            gc_collect_cycles();
            $lev = 1;

            $data['price_markups_general_category'] = $this->showArrayTreeSelect(0, $local_categories, $lev, $options);
        }

        return $data;
    }


    function showArrayTreeSelect($selected_id, $tree, &$level, $options)
    {
        $level_string = '';

        for ($i = 0; $i < $level; $i++) {
            $level_string .= '.';
        }

        foreach ($tree as $branch) {
            $selected = $branch['id'] == $selected_id ? 'selected' : '';
            $options .= "<option $selected value='{$branch['id']}'>$level_string{$branch['name']}</option>\n";
            if (isset($branch['children'])) {
                $level++;
                $options = $this->showArrayTreeSelect($selected_id, $branch['children'], $level, $options);
            } else {
                $level = 1;
            }
        }

        return $options;

    }


    function jobChecker()
    {
        if (file_exists(__DIR__ . "/oc_cli_mod.zip")) {
            unlink(__DIR__ . "/oc_cli_mod.zip");
        }

        gc_enable();

        ini_set('soap.wsdl_cache_enabled', '0');
        ini_set('soap.wsdl_cache_ttl', '0');

        $this->model_extension_for_dealer_import->cancelDeadCommands();
        $this->model_extension_for_dealer_import->deleteOldJobs();

        $get_scheduled_prices = $this->db->query("SELECT price_id FROM " . DB_PREFIX . "4dealer_prices where schedule_update_time=HOUR(NOW()) and local_last_update<4dealer_last_update");

        if ($get_scheduled_prices->num_rows) {
            $prices = [];

            foreach ($get_scheduled_prices->rows as $price) {
                $prices[] = $price['price_id'];
            }
            $ids = implode(',', $prices);
            $check_for_scheduled_running_tasks = $this->db->query(
                "SELECT price_id FROM " . DB_PREFIX . "4dealer_cmd where price_id in($ids) and DATE(create_time)=DATE(NOW()) 
                 and HOUR(Now())=Hour(create_time) and (cmd='full_price_update' or cmd='short_price_update') and status in(1,2,3,4)"
            );

            $runned = [];

            if ($check_for_scheduled_running_tasks->num_rows) {
                foreach ($check_for_scheduled_running_tasks->rows as $price) {
                    $runned[] = $price['price_id'];
                }
            }

            $not_runned_ids = array_diff($prices, $runned);

            if (sizeof($not_runned_ids)) {
                $not_runned_full_price_ids = [];
                $not_runned_short_price_ids = [];

                foreach (array_unique($not_runned_ids) as $priceId) {
                    $price = $this->model_extension_for_dealer_import->getPriceInfoById($priceId);

                    if ($price['meta']['auto_update_type'] == 'full') {
                        $not_runned_full_price_ids[] = $priceId;
                    } else {
                        $not_runned_short_price_ids[] = $priceId;
                    }
                }

                if (sizeof($not_runned_full_price_ids)) {
                    $this->model_extension_for_dealer_import->addJob(['full_price_update' => $not_runned_full_price_ids], ['data_filter' => true], false, 0, 26);
                }

                if (sizeof($not_runned_short_price_ids)) {
                    $this->model_extension_for_dealer_import->addJob(['short_price_update' => $not_runned_short_price_ids], null, false, 0, 25);
                }
            }
        }

        $count_running_jobs = $this->db->query("select count(*) as cnt from " . DB_PREFIX . "4dealer_cmd where status=1");


        if (is_null($count_running_jobs->row) || $count_running_jobs->row['cnt'] <= $this->maxProcessesPerJobs) {
            $get_queue = $this->db->query("SELECT * FROM " . DB_PREFIX . "4dealer_cmd where status=4 order by priority, create_time limit " . $this->maxProcessesPerJobs);

            $jobs_for_run = [];

            if ($get_queue->num_rows) {
                foreach ($get_queue->rows as $job) {

                    // getProductsInfo можно запускать только 1
                    if (in_array('getProductsInfo', array_column($jobs_for_run, 'cmd'))) {
                        continue;
                    }

                    if (method_exists($this, $job['cmd'])) {
                        $this->logger("[{$job['cmd']}] {$job['cmd']} found");
                        $this->model_extension_for_dealer_import->updateJobById(
                            $job['cmd_id'],
                            ['status' => 1, 'pid' => getmypid(), 'start_time' => $this->model_extension_for_dealer_import->getCurrentMysqlTime()]
                        );

                        $jobs_for_run[] = $job;
                    }
                }
            }

            foreach ($jobs_for_run as $job) {
                $this->model_extension_for_dealer_import->updateJobById(
                    $job['cmd_id'],
                    ['status' => 1, 'pid' => getmypid(), 'start_time' => $this->model_extension_for_dealer_import->getCurrentMysqlTime()]
                );

                $this->logger("[{$job['cmd_id']}] update: status => 1, pid => " . getmypid());

                $this->{$job['cmd']}($job);

                $this->logger("[{$job['cmd_id']}] done!");
            }

            exit();
        }

        exit();
    }


    function full_price_update($data, $use_date_filter = false)
    {
        $i = 1;
        $limit = $this->fullUpdateNumRows;
        $error = false;
        $dealer_categories = [];

        $this->logger("[{$data['cmd_id']}] Полное обновление прайса запущено.");


        $this->buildFlatCategoriesList($this->model_extension_for_dealer_import->getRemoteCategories($data['price_id']), $dealer_categories);
        $this->logger("[{$data['cmd_id']}] Получаем список категорий...");
        $this->model_extension_for_dealer_import->setInactiveCategories(array_keys($dealer_categories), $data['price_id']);
        $mappings = $this->model_extension_for_dealer_import->getCategoriesMappingByPriceId($data['price_id']);
        $this->logger("[{$data['cmd_id']}] Получаем связи категорий...");
        $mapping_keys = array_keys($mappings);
        $price_info = $this->model_extension_for_dealer_import->getPriceInfoById($data['price_id']);
        $this->logger("[{$data['cmd_id']}] Получаем информацию о каталоге...");

        if (!is_array($data['data'])) {
            $data['data'] = unserialize($data['data']);
        }
        $date_filter = $use_date_filter || (isset($data['data']['data_filter']) && $data['data']['data_filter']) ? $price_info['local_last_full_update'] : false;
        if ($date_filter) {
            $this->model_extension_for_dealer_import->addJob(['short_price_update' => [$data['price_id']]], null, false, 0, 25);
            $this->logger("[{$data['cmd_id']}] Дополнительно создаем задачу быстрого обновления...");
        }
        $general_data = ['small_quantity_transform' => $price_info['small_quantity_transform'], 'medium_quantity_transform' => $price_info['medium_quantity_transform'], 'large_quantity_transform' => $price_info['large_quantity_transform'], 'price_name' => $price_info['price_name']];

        $not_last_step = true;

        if (empty($mapping_keys)) {
            $this->logger("[{$data['cmd_id']}] Нет ни одной сопоставленной категории для данного дистрибьютора. Пожалуйста, выполните сопоставаление.");
        }

        do {
            $this->logger("[{$data['cmd_id']}] Цикл получения записей товаров {$i}:");
            $liquid = isset($price_info["meta"]["liquid"]) ? $price_info["meta"]["liquid"] : 0 ;
            $avail = $this->config->get('for_dealer_import_settings_avail_cond') ?? 'avail';
            $array_data = $this->model_extension_for_dealer_import->getPriceItems($data['price_id'], $date_filter, false, $i, $mapping_keys, $limit, $liquid, $avail );

            $this->logger("[{$data['cmd_id']}] Запрашиваем список записей...");
            if (isset($array_data['Result']) && empty($array_data['Error'])) {
                if ($array_data['Info']['numRows'] != $limit) {
                    $not_last_step = false;
                    $this->logger("[{$data['cmd_id']}] Отмечаем шаг последним.");
                }

                $this->model_extension_for_dealer_import->addJob(['createProducts' => [$data['price_id']]], [$array_data, $general_data, $price_info, $mappings, $dealer_categories, !$not_last_step, $date_filter], false, $data['cmd_id']);
                $this->logger("[{$data['cmd_id']}] Добавляем задачу create_products...");

                $this->model_extension_for_dealer_import->updateJobById($data['cmd_id'], ['records' => 100 * $i]);
                $this->logger("[{$data['cmd_id']}] Устанавливаем количество обработанных записей:" . 100 * $i);

                $i++;
            } else {
                $error = true;
                $this->logger("[{$data['cmd_id']}] Ошибка получения данных.");
            }

            $this->logger("[{$data['cmd_id']}] Шаг окончен.");
        } while ($error == false && $not_last_step);

        if (!$error) {
            $this->logger("[{$data['cmd_id']}]  Всего обработано:" . 100 * $i);
            $this->model_extension_for_dealer_import->setCommandResult($data['cmd_id'], 100 * $i, $data['price_id'], $date_filter, 'full_price_update');
            $this->logger("[{$data['cmd_id']}]  Обработка окончена!");

            return true;
        }

        $this->model_extension_for_dealer_import->updateJobById($data['cmd_id'], [
            'status' => 3,
            'data' => serialize($array_data['Error']),
            'records' => 100 * $i,
            'end_time' => $this->model_extension_for_dealer_import->getCurrentMysqlTime()
        ]);

        $this->logger("[{$data['cmd_id']}] Ошибка задачи: ");
        $this->logger($array_data['Error']);
        $this->logger("In SOAP: ");
        $this->logger([$data['price_id'], $date_filter, false, $i, $mapping_keys, $limit]);

        return false;
    }


    function short_price_update($data)
    {

        $this->shortUpdateNumRows = 150;
        $product_ids = $this->model_extension_for_dealer_import->getDealerIdsByPriceId($data['price_id']);
        if (!$product_ids) {
            $this->full_price_update($data);
            return true;
        }


        $price_markups = $this->model_extension_for_dealer_import->getCategorizedPriceMarkups($data['price_id']);
        $price_info = $this->model_extension_for_dealer_import->getPriceInfoById($data['price_id']);

        $rows_count = sizeof($product_ids);

        $ids = array_keys($product_ids);

        $iteration_count = ceil($rows_count / $this->shortUpdateNumRows);
        $k = 0;

        $last_step = false;

        for ($i = 0; $i < $iteration_count; $i++) {

            $target_ids = [];
            for ($j = $i * $this->shortUpdateNumRows; $j < ($i + 1) * $this->shortUpdateNumRows && $j < $rows_count; $j++) {
                $target_ids[] = $ids[$j];
                $k++;
            }
            if ($k == $rows_count) {
                $last_step = true;
            }
            $this->model_extension_for_dealer_import->addJob(['updateProducts' => [$data['price_id']]], [$target_ids, $price_markups, $price_info, $last_step], false, $data['cmd_id']);

        }

        //$this->add_job(array('clear_old_products' => array($data->price_id)), array($data->price_id, $data->create_time), false, $data->cmd_id, 54);

        return $this->model_extension_for_dealer_import->setCommandResult($data['cmd_id'], $k, $data['price_id'], false, 'short_price_update');
    }

    function updateProducts($data)
    {
        $target_data = unserialize($data['data']);
        $price_markups = $this->model_extension_for_dealer_import->getCategorizedPriceMarkups($data['price_id']);

        $price_info = $this->model_extension_for_dealer_import->getPriceInfoById($data['price_id']);

        $default_store_id = $this->config->get('config_store_id');
        $hide_dup = $this->config->get('for_dealer_import_settings_hide_dup');
        $hide_dup_cond = $this->config->get('for_dealer_import_settings_hide_dup_cond');
        $sort_type = $hide_dup_cond ? SORT_DESC : SORT_ASC;
        $duplicates = $this->model_extension_for_dealer_import->getDuplicatesData();
        //      $product_ids = $this->model_extension_for_dealer_import->getDealerIdsByPriceId($data['price_id']);

        $k = 0;
        $fetch_products = $this->model_extension_for_dealer_import->getPriceItemsByIdArray($target_data[0]);
        unset($target_data[0]);


        if (isset($fetch_products['Result']) && empty($fetch_products['Error'])) {

            foreach ($fetch_products['Result'] as $product_data) {


                $product = $this->model_extension_for_dealer_import->getProductByForDealerId($product_data['id']);
                $product['quantity'] = $this->model_extension_for_dealer_import->getQuantity($product_data['free'], $target_data[2]['small_quantity_transform'], $target_data[2]['medium_quantity_transform'], $target_data[2]['large_quantity_transform']);
                
                $product['stock_status_id'] = 
                        $product['quantity'] ? 7 
                                             : (($product_data['transit'] != "0" || $product_data['transit_next'] != "0" || $product_data['transit_long'] != "0") ? 8 
                                             : 5);
                
                if ($product['stock_status_id'] == 5) {
                    $product['status'] = 0;
                } else {
                    $product['status'] = 1;
                }
                
                if ( !empty($product_data['transit_next_data']) ) {
                    $product['date_available'] = date('Y-m-d', strtotime($product_data['transit_next_data']));
                }
                
                $product['price'] = $this->model_extension_for_dealer_import->recalculatePrice($target_data[2]['price_markup'], $price_markups, $product_data['price_category_id'], str_replace([' ', ','], ['', '.'], $product_data['my_prices']['price_rub']));

                /** Перерасчет цены с учетом опций и округлений */
                $product['price'] = $this->model_extension_for_dealer_import->checkoutPrice($price_info, $product_data, $product['price']);

                $product['product_store'] = [$default_store_id];
                $product['DuID'] = $product_data['DuID'];
                $this->model_extension_for_dealer_import->editProductWithDealerData($product['product_id'], $product);
                $k++;

                if ($hide_dup && isset($duplicates[$product['DuID']]) && (sizeof($duplicates[$product['DuID']]) > 1 || $duplicates[$product['DuID']][0]['dealer_id'] != $product['dealer_id'])) {
                    $this->hideDuplicates($duplicates, $product, $product['product_id'], $sort_type);
                }
                unset($product, $product_data);

            }

            $this->model_extension_for_dealer_import->processKiller($data['cmd_id'], $k);
            $this->model_extension_for_dealer_import->updateJobById($data['cmd_id'], ['records' => $k]);

        }


        if ($target_data[3]) {
            $this->model_extension_for_dealer_import->addJob(['clearOldProducts' => [$data['price_id']]], [$data['price_id'], $data['create_time']], false, $data['cmd_id'], 54);
            /*$count_running_jobs = $this->db->get_var("select count(*) from {$this->db->prefix}4dealer_cmd where status in (1,4) and cmd='hide_duplicates'");
                if ((is_null($count_running_jobs) || $count_running_jobs == 0) && sizeof($this->getLocalPriceList()) > 1) {
                    $this->add_job(array('hide_duplicates' => array($data->price_id)), null, false, 0, 55);
                }
            */
        }
        return $this->model_extension_for_dealer_import->setCommandResult($data['cmd_id'], $k, $data['price_id'], false, 'updateProducts');
    }

    function clearOldProducts($data)
    {
        $target_data = unserialize($data['data']);
        $this->model_extension_for_dealer_import->clearStockForUnupdatedProducts($target_data[0], $target_data[1]);
        return $this->model_extension_for_dealer_import->setCommandResult($data['cmd_id'], 0, $data['price_id'], false, 'clearOldProducts');
    }


    function createProducts($data)
    {
        $this->load->model('catalog/product');
        $this->load->model('catalog/manufacturer');

        $this->logger("[{$data['cmd_id']}] Запущена задача обработки товаров!");

        $default_store_id = $this->config->get('config_store_id');
        $hide_dup = $this->config->get('for_dealer_import_settings_hide_dup');
        $hide_dup_cond = $this->config->get('for_dealer_import_settings_hide_dup_cond');
        $sort_type = $hide_dup_cond ? SORT_DESC : SORT_ASC;
        $avail = $this->config->get('for_dealer_import_settings_avail_cond') ?? 'avail';
                //isset($price_info["meta"]["avail"]) ? $price_info["meta"]["avail"] : 'avail' ;

        $exclude_str = $this->config->get('for_dealer_import_settings_exclude_str');
        $exclude_str = strlen($exclude_str) > 0 ? explode(',', $exclude_str) : [];

        $target_data = unserialize($data['data']);

        $price_info = $this->model_extension_for_dealer_import->getPriceInfoById($data['price_id']);

        $price_markups = $this->model_extension_for_dealer_import->getCategorizedPriceMarkups($data['price_id']);
        $language_id = $this->model_extension_for_dealer_import->getDefaultLanguageId();
        $duplicates = $this->model_extension_for_dealer_import->getDuplicatesData();
        //var_dump($duplicates);
        $new_products = [];
        $manufacturers = [];

        $j = 0;

        $data_mapper = ['part_no' => 'sku', 'free' => 'quantity', 'name' => 'model', 'price_id' => 'dealer_price_id', 'id' => 'dealer_id', 'DuID' => 'DuID'];

        $empty_data = ['upc' => '', 'ean' => '', 'jan' => '', 'isbn' => '', 'mpn' => '', 'location' => '', 'minimum' => 1, 'subtract' => 1, 'date_available' => '0000-00-00', 'shipping' => 1, 'weight' => 0, 'weight_class_id' => 0, 'length' => 0, 'width' => 0, 'height' => 0, 'length_class_id' => 0, 'status' => 1, 'tax_class_id' => 9, 'sort_order' => 0, 'points' => 0, 'product_store' => [$default_store_id]];

        if (isset($target_data[0]['Result']) && empty($target_data[0]['Error'])) {
            $this->logger("[{$data['cmd_id']}] Товаров для обработки: " . count($target_data[0]['Result']));

            foreach ($target_data[0]['Result'] as $key => $product_data) {
                $this->logger("[{$data['cmd_id']}] Обрабатываем товар:" . $product_data['name']);

                $oc_product_data = [];
                $product_data = array_merge($product_data, $target_data[1]);

                foreach ($data_mapper as $dealer_key => $value) {
                    $oc_product_data[$value] = $product_data[$dealer_key];
                }

                /** Формирование названия модели */
                $part_no_model = ($price_info['meta']['did_as_partno'] == 1 && empty($product_data['part_no']) ) ? $product_data['DuID'] : $product_data['part_no'];
                $oc_product_data['model'] = $price_info['meta']['partno_in_model'] ? $part_no_model : $oc_product_data['model'];

                if (!empty($exclude_str)) {
                    foreach ($exclude_str as $excl) {
                        if (strpos(strtolower($oc_product_data['model']), strtolower(trim($excl))) !== false) {
                            $this->logger("[{$data['cmd_id']}] Найдено стоп-слово '{$excl}'.");
                            continue 2;
                        }
                    }
                }

                $oc_product_data = array_merge($oc_product_data, $empty_data);
                
                $oc_product_data['price'] = $this->model_extension_for_dealer_import->recalculatePrice($target_data[2]['price_markup'], $price_markups, $product_data['price_category_id'], str_replace([' ', ','], ['', '.'], $product_data['my_prices']['price_rub']));

                /** Перерассчет цены с учетом опций и округлений */
                $oc_product_data['price'] = $this->model_extension_for_dealer_import->checkoutPrice($price_info, $product_data, $oc_product_data['price']);


                $oc_product_data['quantity'] = $this->model_extension_for_dealer_import->getQuantity($oc_product_data['quantity'], $product_data['small_quantity_transform'], $product_data['medium_quantity_transform'], $product_data['large_quantity_transform']);

                //$oc_product_data['stock_status_id'] = $oc_product_data['quantity'] ? 7 : 5;
                $oc_product_data['stock_status_id'] = 
                        $oc_product_data['quantity'] ? 7 
                                                     : (($product_data['transit'] != "0" || $product_data['transit_next'] != "0" || $product_data['transit_long'] != "0") ? 8 
                                                     : 5);
                if ($oc_product_data['stock_status_id'] == 5 && $avail != 'all') {
                    $oc_product_data['status'] = 0;
                } else {
                    $oc_product_data['status'] = 1;
                }
                
                $this->logger("[{$data['cmd_id']}] Параметры наличия: ". print_r([
                    'Цена' => $oc_product_data['price'],
                    'Доступно' => $oc_product_data['quantity'],
                    'Отсутствие на складе' => ($oc_product_data['stock_status_id']==7 ? 'В наличии' : ($oc_product_data['stock_status_id']==8 ? 'Предзаказ' : 'Нет в наличии')),
                    'Статус' => ($oc_product_data['status']==1 ? 'Включено' : 'Отключено'),
                ], true));
                
                if ( !empty($product_data['transit_next_data']) ) {
                    $oc_product_data['date_available'] = date('Y-m-d', strtotime($product_data['transit_next_data']));
                }
                
                // preg_match_all('/[\d,]*/',$product_data['my_prices']['price_rub'],$matches);

                $oc_product_data['product_description'] = [
                    $language_id => [
                        'name' => $product_data['name'],
                        'description' => '',
                        'meta_title' => $oc_product_data['model'],
                        'tag' => '',
                        'meta_keyword' => '',
                        'meta_description' => ''
                    ]
                ];


                if (!empty($product_data['brand'])) {

                    if (!isset($manufacturers[$product_data['brand']]) || $manufacturers[$product_data['brand']] == false) {

                        $manufacturer_data = $this->model_extension_for_dealer_import->getManufacturerByName($product_data['brand']);
                        if ($manufacturer_data == false) {
                            $manufacturer_data = [];
                            $manufacturer_data['manufacturer_id'] = 
                                $this->model_catalog_manufacturer->addManufacturer([
                                    'name' => $product_data['brand'],
                                    'sort_order' => 0,
                                    'manufacturer_store' => array($default_store_id)
                                ]);
                        }
                        
                        /* else {
                            $this->model_catalog_manufacturer->editManufacturer($manufacturer_data['manufacturer_id'], array('name' => $product_data['brand'], 'sort_order' => 0, 'manufacturer_store' => array($default_store_id)));
                        }*/
                        
                        $manufacturers[ $product_data['brand'] ] = $manufacturer_data['manufacturer_id'];
                    }

                    if ($manufacturers[ $product_data['brand'] ]) {
                        $oc_product_data['manufacturer_id'] = $manufacturers[ $product_data['brand'] ];
                    }
                }

                $oc_product_data['product_attribute'][] = ['attribute_id' => $this->dealer_no_id, 'product_attribute_description' => [$language_id => ['text' => $product_data['no']]]];

                $oc_product_data['product_attribute'][] = ['attribute_id' => $this->dealer_name_id, 'product_attribute_description' => [$language_id => ['text' => $price_info['price_name']]]];

                $product = $this->model_extension_for_dealer_import->getProductByForDealerId($product_data['id']);

                if (!$product) {
                    /* Add product category for new goods */
                    if (isset($target_data[3][$product_data['price_category_id']])) {
                        $oc_product_data['product_category'] = [$target_data[3][$product_data['price_category_id']]];
                    }
                    //var_dump([ 'model' => $oc_product_data['model'], 'quantity' => $oc_product_data['quantity'], 'stock_status_id' => $oc_product_data['stock_status_id'], 'status' => $oc_product_data['status'], ]);
                    if ($new_product_id = $this->model_extension_for_dealer_import->addProductWithDealerData($oc_product_data)) {
                        $this->logger("[{$data['cmd_id']}] Товар не найден в базе. ({$new_product_id}) - создали.");
                        $j++;
                    }

                } else {
                    $this->logger("[{$data['cmd_id']}] Товар найден в базе ({$product['product_id']}). Обновляем.");
                    $oc_product_data = $this->preSaveName($oc_product_data, $product['product_id'], $data, $price_info, $language_id);
                    /*var_dump([
                        [ 'model' => $product['model'], 'quantity' => $product['quantity'], 'stock_status_id' => $product['stock_status_id'], 'status' => $product['status'], ],
                        [ 'model' => $oc_product_data['model'], 'quantity' => $oc_product_data['quantity'], 'stock_status_id' => $oc_product_data['stock_status_id'], 'status' => $oc_product_data['status'], ]
                    ]);*/
                    $this->model_extension_for_dealer_import->editProductWithDealerData($product['product_id'], $oc_product_data);
                }

                /** @var $new_products - массив, где ключ - product_id, а значение - price_category_id */
                $new_products[$product_data['id']] = $product_data['price_category_id'];
                
                if ($hide_dup && isset($duplicates[$oc_product_data['DuID']]) && (sizeof($duplicates[$oc_product_data['DuID']]) > 1 || $duplicates[$oc_product_data['DuID']][0]['dealer_id'] != $oc_product_data['dealer_id'])) {
                    $this->hideDuplicates($duplicates, $oc_product_data, $product_data['id'], $sort_type,$avail);
                }
                
                unset($oc_product_data, $dealer_key, $value, $product_data, $product, $key);
            }

            $this->model_extension_for_dealer_import->processKiller($data['cmd_id'], $j);

            if (sizeof($new_products)) {
                $this->logger("[{$data['cmd_id']}] Создаем задачу создания карточек новых товаров.");
                $this->model_extension_for_dealer_import->addJob(['getProductsInfo' => [$data['price_id']]], $new_products, false, $data['cmd_id'], 52);
            }

            $this->model_extension_for_dealer_import->updateJobById($data['cmd_id'], ['records' => $j]);

            if ($target_data[5] && !$target_data[6]) {
                $this->logger("[{$data['cmd_id']}] Создаем задачу отключения отсутствующих товаров.");

                $this->model_extension_for_dealer_import->addJob(['clearOldProducts' => [$data['price_id']]], [$data['price_id'], $data['create_time']], false, $data['cmd_id'], 54);
                /* $count_running_jobs = $this->db->query("select count(*) as cnt from " . DB_PREFIX . "4dealer_cmd where status in (1,4) and cmd='hideDuplicates'");

                 if ((is_null($count_running_jobs->row) || $count_running_jobs->row['cnt'] == 0) && sizeof($this->model_extension_for_dealer_import->getLocalPriceList()) > 1) {
                     $this->addJob(array('hideDuplicates' => array($data['price_id'])), null, false, 0, 55);
                 }
                 */
            }

            return $this->model_extension_for_dealer_import->setCommandResult($data['cmd_id'], $j, $data['price_id'], false, 'createProducts');
        }

        $this->model_extension_for_dealer_import->updateJobById($data['cmd_id'], ['status' => 3, 'data' => serialize($target_data[0]['Error']), 'records' => $j, 'end_time' => $this->model_extension_for_dealer_import->getCurrentMysqlTime()]);

        return false;

    }

    function hideDuplicates($duplicates, $oc_product_data, $product_id, $sort_type, $avail='avail')
    {
        $is_present = false;
        foreach ($duplicates[$oc_product_data['DuID']] as $dup_data) {
            if ($dup_data['product_id'] == $product_id) {
                $this->logger("Товар '{$dup_data['model']}' [{$dup_data['DuID']}] найден в списке дублей.");
                $is_present = true;
            }
        }

        if (!$is_present) {
            $duplicates[$oc_product_data['DuID']][] = ['product_id' => $product_id, 'dealer_id' => $oc_product_data['dealer_id'], 'quantity' => $oc_product_data['quantity'], 'price' => $oc_product_data['price'], 'stock_status_id' => $oc_product_data['stock_status_id'], 'model' => $oc_product_data['model'], 'DuID' => $oc_product_data['DuID']];
        }

        sortBySubkey($duplicates[$oc_product_data['DuID']], 'price', $sort_type);

        $target_product_id = 0;

        foreach ($duplicates[$oc_product_data['DuID']] as $dup) {
            switch (true) {
                case ($dup['quantity'] > 0):
                    $target_product_id = $dup['product_id'];
                    break;
                case ($dup['stock_status_id'] == 8 && $avail == 'transit'):
                    $target_product_id = $dup['product_id'];
                    break;
                case ($avail == 'all'):
                    $target_product_id = $dup['product_id'];
                    break;
            }
            if ($target_product_id > 0) {                
                break;
            }
        }
        
        $this->db->query("update " . DB_PREFIX . "product SET status=0 where DuID={$oc_product_data['DuID']}");
        $this->logger("Товары c '{$dup_data['model']}' [{$dup_data['DuID']}] отключены.");
        if ( $target_product_id ) {
            $this->logger("Товар c ID '{$target_product_id}' активирован.");
            $this->db->query("update " . DB_PREFIX . "product SET status=1 where product_id=$target_product_id limit 1");
        }
        //$this->db->query("update " . DB_PREFIX . "product SET status=0 where product_id!=$target_product_id and DuID={$oc_product_data['DuID']}");
        //$this->db->query("update " . DB_PREFIX . "product SET status=1 where product_id=$target_product_id limit 1");
    }

    function getProductsInfo($data)
    {
        $j = 0;
        $this->logger("[{$data['cmd_id']}] Задача загрузки карточек товаров запущена.");

        $language_id = $this->model_extension_for_dealer_import->getDefaultLanguageId();
        $price_info = $this->model_extension_for_dealer_import->getPriceInfoById($data['price_id']);

        /** @var $jobData - массив, где ключ - product_id, а значение - price_category_id */
        $jobData = unserialize($data['data']);
        $jobProductIds = array_keys($jobData);

        $fetch_product_info = $this->model_extension_for_dealer_import->getPriceItemInfoByIdArray($jobProductIds, $price_info['other_images'] ? 'Y' : false);

        if (isset($fetch_product_info['Result']) && empty($fetch_product_info['Error'])) {

            foreach ($fetch_product_info['Result'] as $product_info) {
                $this->logger("[{$data['cmd_id']}] Карточка для {$product_info['name']}.");

                $images = [];
                $product = $this->model_extension_for_dealer_import->getProductByForDealerId($product_info['id']);

                if (!$product) {
                    continue;
                }

                $j++;
                $options_string = '';

                if (isset($product_info['info'])) {
                    $options = json_decode($product_info['info']);

                    if (isset($options)) {
                        $this->logger("[{$data['cmd_id']}] Параметры найдены.");

                        $options_string .= "<table>\n<tr>\n<td>Характеристика</td>\n<td>Значение</td>\n</tr>\n";

                        /** Фильтруем опции, меняем значение в соответствии с заменами значений */

                        $dealer_cat = $jobData[$product['dealer_id']];

                        $priceInfo = $this->model_extension_for_dealer_import->getPriceInfoById($product['dealer_price_id'], true);
                        $optionsMerge = $this->model_extension_for_dealer_import->getOptionsMergeByPriceId($product['dealer_price_id'], $dealer_cat, true);

                        $options = $this->model_extension_for_dealer_import->filterOptions($priceInfo, $options, $optionsMerge);

                        foreach ($options as $option) {
                            if (!isset($option->name) || !isset($option->value)) {
                                continue;
                            }

                            $attrId = $this->model_extension_for_dealer_import->addNewAttributeOrGet($option->name);

                            if (!$attrId)
                                continue;

                            $product['product_attribute'][] = ['attribute_id' => $attrId, 'product_attribute_description' => [$language_id => ['text' => $option->value]]];

                            $options_string .= "<tr>\n";
                            $options_string .= "<td>" . $option->name . "</td>\n<td>" . $option->value . "</td>\n";
                            $options_string .= "</tr>\n";
                        }

                        $options_string .= "</table>\n";

                        if ($price_info['product_add_table'] == 0) {
                            $options_string = '';
                        }

                        $this->logger("[{$data['cmd_id']}] Параметры обработаны.");
                    } else {
                        $this->logger("[{$data['cmd_id']}] Параметры товара отсутствуют");
                    }
                }

                if ($options_string || $product_info['description']) {
                    $product['product_description'][$language_id] = $this->model_extension_for_dealer_import->makeProductDescription($product['product_id'], $language_id, [
                        'name'          => $product_info['name'],
                        'part_no'       => $product_info['part_no'],
                        'description'   => $product_info['description'] . "\n" . $options_string,
                    ]);
                    
                    $product = $this->preSaveName($product, $product['product_id'], $data, $price_info, $language_id);
                }

                if ($product_info['weight']) {
                    $product['weight'] = $product_info['weight'];
                }
                
                if ($product_info['dimention_l']) {
                    $product['length'] = $product_info['dimention_l'];
                }
                
                if ($product_info['dimention_w']) {
                    $product['width'] = $product_info['dimention_w'];
                }
                
                if ($product_info['dimention_h']) {
                    $product['height'] = $product_info['dimention_h'];
                }

                if (isset($product_info['image']) && is_array($product_info['image'])) {

                    $size = sizeof($product_info['image']);
                    $main_image = false;

                    if ($size) {
                        $i = 0;
                        $this->logger("[{$data['cmd_id']}] Картинки найдены. Грузим картинки.");

                        foreach ($product_info['image'] as $image_url) {
                            $local_image = $this->model_extension_for_dealer_import->downloadImage($image_url);

                            if ($local_image) {
                                if ($i)
                                    $images[$i] = ['image' => self::IMAGE_PATH . $local_image, 'sort_order' => 0]; else $main_image = self::IMAGE_PATH . $local_image;

                                $i++;
                            }
                        }

                        if (!is_null($main_image) && $main_image) {
                            $product['image'] = $main_image;

                            if ($i > 1) {
                                $product['product_image'] = $images;
                            }
                        }

                    } else {
                        $this->logger("[{$data['cmd_id']}] Картинки отсутствуют.");
                    }
                }

                /*var_dump([
                    [ 'model' => $product['model'], 'quantity' => $product['quantity'], 'stock_status_id' => $product['stock_status_id'], 'status' => $product['status'], ]
                ]);*/

                $this->model_extension_for_dealer_import->editProductWithDealerData($product['product_id'], $product);
            }

            $this->db->query("UPDATE " . DB_PREFIX . "4dealer_cmd set status = 2, records = $j,end_time = '" . $this->model_extension_for_dealer_import->getCurrentMysqlTime() . "' 
                 WHERE cmd_id = {$data['cmd_id']}");

            return true;
        }

        $this->db->query("UPDATE " . DB_PREFIX . "4dealer_cmd SET status = 3, data ='" . $this->db->escape(serialize($fetch_product_info['Error'])) . "',
             records = $j, end_time = '" . $this->model_extension_for_dealer_import->getCurrentMysqlTime() . "' where cmd_id = {$data['cmd_id']}");

        return false;
    }

    function preSaveName($product, $local_product_id, $data, $price_info, $language_id)
    {
        if ($price_info['meta']['overwrite_product_name'] == 1) {
            if (isset($local_product_id)) {
                $this->logger("[{$data['cmd_id']}] Для товара ({$local_product_id}) обновляем название (перетираем старое)");
            } else {
                $this->logger("[{$data['cmd_id']}] WTF");
            }

            return $product;
        }

        $productOldDescription = $this->model_extension_for_dealer_import->getProductOldDescription($local_product_id, $language_id);

        if (!$productOldDescription) {
            $this->logger("[{$data['cmd_id']}] productOldDescription -  не найдено");

            return $product;
        }

        // Сохраняем старое описание (если старое не пустое или новое пустое)
        if (trim($productOldDescription['description']) || !trim($product['product_description'][ $language_id ]['description'])) {
            $product['product_description'][ $language_id ]['description'] = $productOldDescription['description'];
            $this->logger("[{$data['cmd_id']}] Для товара ({$local_product_id}) восстанавливаем описание, новое - пустое");
        }

        // Сохраняем старое название товара
        $product['product_description'][ $language_id ]['name'] = $productOldDescription['name'];

        $this->logger("[{$data['cmd_id']}] Для товара ({$local_product_id}) сохраняем старое название: " . $productOldDescription['name']);

        return $product;
    }

    function logger($msg) {
        $this->model_extension_for_dealer_import->logger($msg);
    }

}
