<?php

/**
 * Class ModelToolForDealerImport
 */

static $registry = null;

require_once 'for_dealer_table_manager.php';

class ModelToolForDealerImport extends Model
{
    /**
     * Текущая версия модуля (указывается в 2-х местах, здесь и в install.xml)
     */
    public $version = '1.6.1';

    private $error = [];
    protected $null_array = [];
    protected $use_table_seo_url = false;
    protected $soap;
    private $options;

    private $defaultPriceMeta = [
        'auto_update_type' => 'full',
        'partno_in_model' => 0,
        'did_as_partno' => 0,
        'avail' => 'avail',
        'liquid' => 0,
        'overwrite_product_name' => 1,
    ];

    const ERR_CODE_EMPTY_DATA = 103;
    const SOAP_ADDRESS = 'https://4dealer.ru/soap/?WSDL';
    protected $images_dir;

    protected $storeDir;

    public $QB;

    private $cacheSpace = [];

    public $localActualVersionFile;

    private $loggerEnabled;

    public function __construct($registry)
    {
        parent::__construct($registry);

        $this->QB = new QueryBuilderForDealer($this->db);

        $this->storeDir = DIR_APPLICATION . 'store/4dealer';

        $this->use_table_seo_url = version_compare(VERSION, '3.0', '>=') ? true : false;
        
        if (\PHP_VERSION_ID < 80000) {
            libxml_disable_entity_loader(false);
        }

        $this->soap = new SoapClient(self::SOAP_ADDRESS, ['exceptions' => 0, 'cache_wsdl' => WSDL_CACHE_NONE, 'trace' => 1,]);
        $this->options = [
            'login' => $this->config->get('for_dealer_import_settings_login'),
            'key' => $this->key = $this->config->get('for_dealer_import_settings_token')
        ];

        $this->loggerEnabled = $this->config->get('for_dealer_import_settings_logger');

        $this->images_dir = DIR_IMAGE . "catalog/4dealer/";

        $this->localActualVersionFile = $this->storeDir . '/oc3_version.txt';

        $this->checkVersionUpdateProcedure($this->version);
        $this->remoteUpdatePluginVersion();
    }


    private function getDataFields($fields)
    {
        $data = [];

        foreach ($fields as $key => $field) {
            if (!isset($_REQUEST[$key]) || (empty($_REQUEST[$key]) && $_REQUEST[$key] != 0)) {
                // return false;
            }

            $data[$field] = $_REQUEST[$key];
        }

        return $data;
    }

    /**
     * Проверка были ли выполнены миграции для версии
     * Флагом является существование файла в папке 4dealer-store
     *
     * @param $version
     */
    public function checkVersionUpdateProcedure($version)
    {
        $fileVersion = $this->storeDir . '/updated.at.v' . $version . '.txt';

        if (@file_exists($fileVersion)) {
            return;
        }

        $this->runVersionUpdateProcedure($version);

        @mkdir($this->storeDir, 0777, true);
        file_put_contents($fileVersion, $version);
    }

    /**
     * Remote check plugin version once per day
     * Based on checking create file time
     */
    function remoteUpdatePluginVersion()
    {
        $lastUpdate = @filemtime($this->localActualVersionFile);
        $remoteFile = 'https://4dealer.ru/files/oc3_version.txt';

        if (date('D m Y') == date('D m Y', $lastUpdate)) {
            return;
        }

        file_put_contents($this->localActualVersionFile, file_get_contents($remoteFile));
    }

    /**
     * Get actual version from local file on every page refresh
     */
    function actualPluginVersion()
    {
        return trim(file_get_contents($this->localActualVersionFile));
    }

    function isActualVersion()
    {
        return version_compare($this->actualPluginVersion(), $this->version, '<=');
    }

    public function runVersionUpdateProcedure($version)
    {
        $version = (int)preg_replace('/\D/', '', $version);

        /** Проверка существования таблиц модуля */
        $isCreatedSchemas = $this->checkColumn('4dealer_prices', 'price_id');

        if ($isCreatedSchemas && $isCreatedSchemas->row['cnt'] == 0) return;

        /** Проверка версии 1.4.1 */
        $checkProcedure = $this->checkColumn('4dealer_price_markup', 'id');

        if ($checkProcedure && $checkProcedure->row['cnt'] == 0) {

            $this->db->query("DELETE FROM " . DB_PREFIX . "4dealer_categories_term WHERE term_id NOT IN (
                    SELECT category_id FROM " . DB_PREFIX . "category
                )");

            $this->db->query("ALTER TABLE " . DB_PREFIX . "4dealer_prices ADD `round_price` INT NOT NULL");
            $this->db->query("ALTER TABLE " . DB_PREFIX . "4dealer_prices ADD `round_rule` INT NOT NULL");
            $this->db->query("ALTER TABLE " . DB_PREFIX . "4dealer_prices ADD `meta` TEXT NOT NULL");

            $this->db->query("ALTER TABLE " . DB_PREFIX . "4dealer_prices ADD `product_add_table` INT NOT NULL");
            $this->db->query("ALTER TABLE " . DB_PREFIX . "4dealer_prices ADD `auto_load_options` INT NOT NULL");

            $this->db->query("ALTER TABLE " . DB_PREFIX . "4dealer_price_markup DROP PRIMARY KEY");
            $this->db->query("ALTER TABLE `" . DB_PREFIX . "4dealer_price_markup` ADD `id` INT NOT NULL AUTO_INCREMENT FIRST, ADD PRIMARY KEY (`id`)");
            $this->db->query("ALTER TABLE " . DB_PREFIX . "4dealer_price_markup ADD `price_is_rrc` INT NOT NULL AFTER `markup`");

            $this->db->query("
                CREATE TABLE IF NOT EXISTS " . DB_PREFIX . "4dealer_options_merge (
                    `id` INT NOT NULL AUTO_INCREMENT,
                    `4dealer_price_id` INT NOT NULL,
                    `section_id` INT NOT NULL,
                    `option_name` VARCHAR(255) NOT NULL,
                    `changed_option_name` VARCHAR(255) NOT NULL,
                    `replaces` TEXT NOT NULL,
                    PRIMARY KEY (`id`),
                    INDEX (`4dealer_price_id`)
                ) DEFAULT CHARSET=utf8 ENGINE=InnoDB"
            );

        }

    }

    public function getDefaultLanguageId()
    {
        if (isset($this->defaultLanguageId)) {
            return $this->defaultLanguageId;
        }

        $code = $this->config->get('config_language');

        $sql = "SELECT language_id FROM `" . DB_PREFIX . "language` WHERE code = '" . $this->db->escape($code) . "'";
        $result = $this->db->query($sql);

        $language_id = 1;

        if ($result->rows) {
            foreach ($result->rows as $row) {
                $language_id = $row['language_id'];
                break;
            }
        }

        return $this->defaultLanguageId = $language_id;
    }

    public function getLanguages()
    {
        $query = $this->db->query("SELECT * FROM `" . DB_PREFIX . "language` WHERE `status`=1 ORDER BY `code`");
        return $query->rows;
    }

    protected function getCategoryDescriptions(&$languages, $offset = null, $rows = null, $min_id = null, $max_id = null)
    {
        // query the category_description table for each language
        $category_descriptions = array();

        foreach ($languages as $language) {
            $language_id = $language['language_id'];
            $language_code = $language['code'];

            $sql = "SELECT c.category_id, cd.* ";
            $sql .= "FROM `" . DB_PREFIX . "category` c ";
            $sql .= "LEFT JOIN `" . DB_PREFIX . "category_description` cd ON cd.category_id=c.category_id AND cd.language_id='" . (int)$language_id . "' ";

            if (isset($min_id) && isset($max_id)) {
                $sql .= "WHERE c.category_id BETWEEN $min_id AND $max_id ";
            }

            $sql .= "GROUP BY c.`category_id` ";
            $sql .= "ORDER BY c.`category_id` ASC ";

            if (isset($offset) && isset($rows)) {
                $sql .= "LIMIT $offset,$rows; ";
            } else {
                $sql .= "; ";
            }

            $query = $this->db->query($sql);
            $category_descriptions[$language_code] = $query->rows;
        }

        return $category_descriptions;
    }

    public function getCategories($offset = null, $rows = null, $min_id = null, $max_id = null)
    {
        $languages = $this->getLanguages();
        $sql = "SHOW COLUMNS FROM `" . DB_PREFIX . "category_description` LIKE 'meta_title'";
        $query = $this->db->query($sql);
        $exist_meta_title = ($query->num_rows > 0) ? true : false;
        $exist_seo_url_table = $this->use_table_seo_url;

        if ($exist_seo_url_table) {
            $sql = "SELECT c.* FROM `" . DB_PREFIX . "category` c ";
        } else {
            $sql = "SELECT c.*, ua.keyword FROM `" . DB_PREFIX . "category` c ";
            $sql .= "LEFT JOIN `" . DB_PREFIX . "url_alias` ua ON ua.query=CONCAT('category_id=',c.category_id) ";
        }
        if (isset($min_id) && isset($max_id)) {
            $sql .= "WHERE c.category_id BETWEEN $min_id AND $max_id ";
        }
        $sql .= "GROUP BY c.`category_id` ";
        $sql .= "ORDER BY c.`category_id` ASC ";
        if (isset($offset) && isset($rows)) {
            $sql .= "LIMIT $offset,$rows; ";
        } else {
            $sql .= "; ";
        }
        $results = $this->db->query($sql);
        $category_descriptions = $this->getCategoryDescriptions($languages, $offset, $rows, $min_id, $max_id);

        foreach ($languages as $language) {
            $language_code = $language['code'];
            foreach ($results->rows as $key => $row) {
                if (isset($category_descriptions[$language_code][$key])) {
                    $results->rows[$key]['name'][$language_code] = $category_descriptions[$language_code][$key]['name'];
                    $results->rows[$key]['description'][$language_code] = $category_descriptions[$language_code][$key]['description'];
                    if ($exist_meta_title) {
                        $results->rows[$key]['meta_title'][$language_code] = $category_descriptions[$language_code][$key]['meta_title'];
                    }
                    $results->rows[$key]['meta_description'][$language_code] = $category_descriptions[$language_code][$key]['meta_description'];
                    $results->rows[$key]['meta_keyword'][$language_code] = $category_descriptions[$language_code][$key]['meta_keyword'];
                } else {
                    $results->rows[$key]['name'][$language_code] = '';
                    $results->rows[$key]['description'][$language_code] = '';
                    if ($exist_meta_title) {
                        $results->rows[$key]['meta_title'][$language_code] = '';
                    }
                    $results->rows[$key]['meta_description'][$language_code] = '';
                    $results->rows[$key]['meta_keyword'][$language_code] = '';
                }
            }
        }

        $categories = $results->rows;

        $this->sortArrayLocalCategoriesTree($categories);

        return $categories;
    }

    function getDealerIdsByPriceId($price_id)
    {
        $get_ids = $this->db->query("select product_id, dealer_id from " . DB_PREFIX . "product  where dealer_price_id=" . (int)$price_id);
        if ($get_ids->num_rows) {
            foreach ($get_ids->rows as $data) {
                $ids[$data['dealer_id']] = $data['product_id'];
            }
            return $ids;
        }
        return false;

    }

    function clearStockForUnupdatedProducts($price_id, $date, $stock_status_id = 5, $status = 0)
    {
        $get_ids = $this->db->query("select p.product_id, p.model, d.name from " . DB_PREFIX . "product p left join " . DB_PREFIX . "product_description d on (p.product_id = d.product_id) where dealer_price_id=$price_id and date_modified < '$date'");
        if ($get_ids->num_rows) {
            foreach ($get_ids->rows as $data) {
                $this->logger('Отключаем старый товар: '.$data['name'].' ['.$data['model'].']');
            }
        }
        
        
        $this->db->query("update " . DB_PREFIX . "product set quantity=0, stock_status_id=$stock_status_id, status=$status where dealer_price_id=$price_id and date_modified < '$date'");

        $this->logger('clearStockForUnupdatedProducts');
    }

    public function getCommonCatalogStructure($tree = true)
    {
        $result = (array)$this->soap->getCommonCatalogStructure($this->options['login'], md5($this->options['login'] . 'getCommonCatalogStructure' . md5($this->options['key'])), 'N');

        if (empty($result['Error'])) {
            return $tree ? $this->buildTree($result['Result']) : $result['Result'];
        }

        return array();
    }

    public function getPriceStructure($price, $tree = true)
    {
        $result = (array)$this->soap->getPriceStructure($this->options['login'], md5($this->options['login'] . 'getPriceStructure' . md5($this->options['key'])), $price);
        if (empty($result['Error'])) {
            return $tree ? $this->buildTree($result['Result']) : $result['Result'];
        }

        return array();
    }

    public function getPricesList()
    {
        if (!is_null($this->options['login']) && !is_null($this->options['key'])) {
            return (array)$this->soap->getPricesList($this->options['login'], md5($this->options['login'] . 'getPricesList' . md5($this->options['key'])), 'N');
        }
        return [];
    }

    /**
     * Get options list from 4dealer
     *
     * @param array $sectionIds
     * @param bool $ajax
     * @return array|null
     */
    function get_list_options_merge($sectionIds = null, $ajax = true)
    {
        $sectionIds = $sectionIds ? $sectionIds : [$_REQUEST['section_id']];

        $result = $this->soap->getOptionsBySectionIdArray(
            $this->options['login'],
            md5($this->options['login'] . 'getOptionsBySectionIdArray' . md5($this->options['key'])),
            $sectionIds
        );

        if ($ajax != true)
            return $result;

        echo json_encode($result, JSON_UNESCAPED_UNICODE);
    }

    public function getPriceItems($price, $start_date = false, $last_update = false, $offset = 1, $sections = array(), $limit = 100, $liquid = 0, $avail = 'avail' )
    {
        $params = [
            'only_with_prices' => 'Y',
            'limit' => $limit,
            'offset' => $offset,
            'avail' => $avail
        ];

        if (!empty($sections)) {
            $params['sections'] = $sections;
        }

        if ($last_update) {
            $params['last_update'] = $last_update;
        }

        if ($start_date) {
            $params['start_date'] = $start_date;
        }

        return (array)$this->soap->getPriceItems($this->options['login'], md5($this->options['login'] . 'getPriceItems' . md5($this->options['key'])), $price, $params);
    }

    public function getPriceItemsByIdArray($targets)
    {
        if (!is_array($targets)) {
            return false;
        }

        return (array)$this->soap->getPriceItemByIdArray($this->options['login'], md5($this->options['login'] . 'getPriceItemByIdArray' . md5($this->options['key'])), $targets);
    }

    public function getPriceItemInfoByIdArray($id, $other_images = false)
    {
        return (array)$this->soap->getPriceItemInfoByIdArray($this->options['login'], md5($this->options['login'] . 'getPriceItemInfoByIdArray' . md5($this->options['key'])), $id, $other_images);
    }

    public function showError($error)
    {
        if (!empty($error)) {
            var_dump($error);
        }
    }

    public function buildTree(&$elements, $parentId = 0, $parents = array())
    {
        $branch = array();

        if (count($elements) > 0) {
            foreach ($elements as $key => $element) {
                if ($element['parent'] != $parentId) {
                    continue;
                }
                if (!in_array($parentId, $parents)) {
                    $parents[] = $parentId;
                }
                $element['parents'] = $parents;
                $children = $this->buildTree($elements, $element['id'], $parents);
                if (!empty($children)) {
                    $element['children'] = $children;
                }
                unset($children);
                $branch[$element['id']] = $element;
            }
        }

        unset($key, $element);
        $parents = array();
        return $branch;
    }

    public function createSchemas()
    {
        $statuses_table_name = DB_PREFIX . '4dealer_statuses';
        $sql = "
        CREATE TABLE IF NOT EXISTS $statuses_table_name (
            `status_id` tinyint(3) unsigned NOT NULL AUTO_INCREMENT,
            `status_name` varchar(100) NOT NULL,
            PRIMARY KEY (`status_id`)
        ) DEFAULT CHARSET=utf8 ENGINE=InnoDB;
                
        CREATE TABLE IF NOT EXISTS " . DB_PREFIX . "4dealer_prices (
            `price_id` int(11) unsigned NOT NULL DEFAULT '0',
            `price_name` varchar(200)  NOT NULL,
            `local_last_update` datetime DEFAULT '1970-01-01 00:00:00',
            `local_last_full_update` datetime NOT NULL DEFAULT '1970-01-01 00:00:00',
            `4dealer_last_update` datetime DEFAULT '1970-01-01 00:00:00',
            `subscription_end` datetime NOT NULL,
            `categories` longtext,
            `active` tinyint(4) NOT NULL DEFAULT '1',
            `price_markup` smallint(5) unsigned DEFAULT '0',
            `small_quantity_transform` smallint(5) unsigned DEFAULT '1',
            `medium_quantity_transform` smallint(5) unsigned DEFAULT '10',
            `large_quantity_transform` smallint(5) unsigned DEFAULT '100',
            `other_images` tinyint(4) NOT NULL DEFAULT '0',                
            `schedule_update_time` tinyint(3) NOT NULL DEFAULT '-1',
            `round_price` int(10) NOT NULL,
            `round_rule` int(10) unsigned NOT NULL,
            `product_add_table` int(10) unsigned NOT NULL,
            `auto_load_options` tinyint(4) unsigned NOT NULL,
            `meta` TEXT NOT NULL,
            PRIMARY KEY (`price_id`),
            KEY `local_update_idx` (`local_last_update`),
            KEY `remote_update_idx` (`4dealer_last_update`),
            KEY `local_full_update_idx` (    `local_last_full_update`)
        ) DEFAULT CHARSET=utf8 ENGINE=InnoDB;

        CREATE TABLE IF NOT EXISTS " . DB_PREFIX . "4dealer_categories_term (
            `term_id` int(11) unsigned NOT NULL,
            `4dealer_cat_id` int(11) unsigned NOT NULL,
            `4dealer_price_id` int(11) unsigned DEFAULT NULL,
            `is_active` tinyint(3) unsigned NOT NULL DEFAULT '1',
            PRIMARY KEY (`term_id`,`4dealer_cat_id`),
            KEY `price_id` (`4dealer_price_id`),
            KEY `active` (`is_active`)
        ) DEFAULT CHARSET=utf8 ENGINE=InnoDB;

        CREATE TABLE IF NOT EXISTS " . DB_PREFIX . "4dealer_price_markup (
            `id` INT NOT NULL AUTO_INCREMENT,
            `price_id` int(10) unsigned NOT NULL,
            `category_id` bigint(20) unsigned NOT NULL DEFAULT '0',
            `price_low` int(10) unsigned NOT NULL,
            `price_high` int(10) unsigned NOT NULL,
            `markup` int(10) unsigned NOT NULL,
            `price_is_rrc` int(10) unsigned NOT NULL,            
            PRIMARY KEY (`id`)
        ) DEFAULT CHARSET=utf8 ENGINE=InnoDB;

        CREATE TABLE IF NOT EXISTS " . DB_PREFIX . "4dealer_options_merge (
            `id` INT NOT NULL AUTO_INCREMENT,
            `4dealer_price_id` INT NOT NULL,
            `section_id` INT NOT NULL,
            `option_name` VARCHAR(255) NOT NULL,
            `changed_option_name` VARCHAR(255) NOT NULL,
            `replaces` TEXT NOT NULL,
            PRIMARY KEY (`id`),
            INDEX (`4dealer_price_id`)
        ) DEFAULT CHARSET=utf8 ENGINE=InnoDB";

        foreach (explode(';', $sql) as $query) {
            $query = trim($query);
            if (!$query) {
                continue;
            }
            $this->db->query($query);
        }

        $check_status_data = $this->db->query("select count(*) as cnt from $statuses_table_name");
        if ($check_status_data->num_rows) {
            if ($check_status_data->row['cnt'] == 0) {
                $this->db->query("insert into " . DB_PREFIX . "4dealer_statuses(status_name) values('Running')");
                $this->db->query("insert into " . DB_PREFIX . "4dealer_statuses(status_name) values('Finished')");
                $this->db->query("insert into " . DB_PREFIX . "4dealer_statuses(status_name) values('Error')");
                $this->db->query("insert into " . DB_PREFIX . "4dealer_statuses(status_name) values('Waiting')");
                $this->db->query("insert into " . DB_PREFIX . "4dealer_statuses(status_name) values('Canceled')");
                $this->db->query("ALTER TABLE " . DB_PREFIX . "product  
                                  ADD COLUMN `dealer_id` BIGINT UNSIGNED NULL AFTER `date_modified`,
                                  ADD COLUMN `dealer_price_id` SMALLINT UNSIGNED NULL AFTER `dealer_id`,
                                  ADD UNIQUE INDEX `dealer_id_UNIQUE` (`dealer_id` ASC),
                                  ADD UNIQUE INDEX `dealerid_priceid_idx` (`dealer_id` ASC, `dealer_price_id` ASC)");
            }
        }

        $check_other_images_column = $this->checkColumn('4dealer_prices', 'other_images');

        if ($check_other_images_column) {
            if ($check_other_images_column->row['cnt'] == 0) {
                $this->db->query(
                    "ALTER TABLE " . DB_PREFIX . "4dealer_prices ADD COLUMN `other_images` tinyint(4) NOT NULL DEFAULT '0' AFTER `large_quantity_transform`"
                );
            }
        }

        $check_other_images_column = $this->checkColumn('4dealer_prices', 'exclude_str');

        if ($check_other_images_column) {
            if ($check_other_images_column->row['cnt'] == 0) {
                $this->db->query(
                    "ALTER TABLE " . DB_PREFIX . "4dealer_prices ADD COLUMN `exclude_str` text NOT NULL AFTER `other_images`"
                );
            }
        }

        $check_duid_column = $this->checkColumn('product', 'DuID');

        if ($check_duid_column) {
            if ($check_duid_column->row['cnt'] == 0) {
                $this->db->query(
                    "ALTER TABLE " . DB_PREFIX . "product ADD COLUMN `DuID` INT UNSIGNED NOT NULL DEFAULT '0' AFTER `dealer_price_id`, ADD INDEX `duid_idx` (`DuId` ASC)"
                );
            }
        }


        $this->db->query("
        CREATE TABLE IF NOT EXISTS " . DB_PREFIX . "4dealer_cmd (
            `cmd_id` int(11) unsigned NOT NULL AUTO_INCREMENT,
            `parent_id` bigint(20) unsigned DEFAULT '0',
            `cmd` varchar(100) NOT NULL,
            `status` tinyint(3) unsigned NOT NULL DEFAULT '4',
            `create_time` datetime DEFAULT NULL,
            `start_time` datetime NOT NULL,
            `end_time` datetime DEFAULT '1970-01-01 00:00:00',
            `price_id` int(10) unsigned NOT NULL,
            `records` int(10) unsigned DEFAULT '0',
            `data` longtext COLLATE utf8_unicode_ci,
            `pid` int(10) unsigned DEFAULT NULL,
            `priority` tinyint(3) unsigned NOT NULL DEFAULT '50',
            PRIMARY KEY (`cmd_id`),
            KEY `time_idx` (`start_time`,`end_time`),
            KEY `cmd_idx` (`cmd`),
            KEY `fk_wp_4dealer_cmd_1_idx` (`price_id`),
            KEY `fk_wp_4dealer_cmd_2_idx` (`status`),
            KEY `parent_idx` (`parent_id`),
            KEY `priority_idx` (`priority`),
            CONSTRAINT `fk_wp_4dealer_cmd_1` FOREIGN KEY (`price_id`) REFERENCES `" . DB_PREFIX . "4dealer_prices` (`price_id`) ON DELETE NO ACTION ON UPDATE NO ACTION,
            CONSTRAINT `fk_wp_4dealer_cmd_2` FOREIGN KEY (`status`) REFERENCES `" . DB_PREFIX . "4dealer_statuses` (`status_id`) ON DELETE NO ACTION ON UPDATE NO ACTION
        ) DEFAULT CHARSET=utf8 ENGINE=InnoDB");


        if (!is_dir($this->images_dir)) {
            mkdir($this->images_dir);
        }

        $this->load->model('catalog/attribute_group');
        $this->load->model('catalog/attribute');

        $this->addNewAttributeOrGet('Поставщик');
        $this->addNewAttributeOrGet('Номер товара в ИС поставщика');

    }

    public function checkColumn($db_name, $column_name)
    {
        return $this->db->query("SELECT count(column_name) as cnt 
            FROM INFORMATION_SCHEMA.COLUMNS 
            WHERE TABLE_SCHEMA='" . DB_DATABASE . "' AND TABLE_NAME='" . DB_PREFIX . "$db_name' 
            and column_name='$column_name'");
    }

    function getTableEngine($table)
    {
        $sql = "SELECT ENGINE FROM INFORMATION_SCHEMA.TABLES WHERE TABLE_NAME = '{$table}'";

        $res = $this->db->query($sql);
        return $res->rows[0]->ENGINE;
    }


    public function getModuleAttributeGroupId()
    {
        if (isset($this->moduleAttributeGroupId)) {
            return $this->moduleAttributeGroupId;
        }
        
        $this->load->model('catalog/attribute_group');

        $language_id = $this->getDefaultLanguageId();

        $check_attribute_group = $this->db->query(
            "SELECT attribute_group_id FROM " . DB_PREFIX . "attribute_group_description WHERE name = 'Свойства модуля 4dealer'"
        );

        if ($check_attribute_group->num_rows) {
            $attribute_group_id = $check_attribute_group->row['attribute_group_id'];

        } else {
            $attribute_group_id = $this->model_catalog_attribute_group->addAttributeGroup([
                'sort_order' => 0,
                'attribute_group_description' => [$language_id => ['name' => 'Свойства модуля 4dealer']]
            ]);
        }

        return $this->moduleAttributeGroupId = $attribute_group_id;
    }

    public function addNewAttributeOrGet($name, $language_id = null)
    {
        if (!$language_id) {
            $language_id = $this->getDefaultLanguageId();
        }

        $this->load->model('catalog/attribute');

        $attribute_group_id = $this->getModuleAttributeGroupId();

        if (!$attribute_group_id) return false;
        
        if (mb_strlen($name) > 64) {
            $name = mb_substr($name, 0, 64);
        }
        
        $check_attribute = $this->db->query(
            "SELECT * FROM " . DB_PREFIX . "attribute t1, " . DB_PREFIX . "attribute_description t2  
             WHERE t1.attribute_id = t2.attribute_id AND attribute_group_id = '{$attribute_group_id}' AND name='{$name}'"
        );

        if (!$check_attribute->num_rows) {
            return $this->model_catalog_attribute->addAttribute([
                'sort_order' => 0,
                'attribute_group_id' => $attribute_group_id,
                'attribute_description' => [$language_id => ['name' => $name]]
            ]);

        } else {
            return $check_attribute->row['attribute_id'];
        }
    }



    public function getForDealerAttributes()
    {
        $get_attributes = $this->db->query(
            "SELECT attribute_id, name FROM " . DB_PREFIX . "attribute_description 
             WHERE name IN ('Поставщик', 'Номер товара в ИС поставщика') AND language_id = " . $this->getDefaultLanguageId()
        );

        if ($get_attributes->num_rows) {
            return $get_attributes->rows;
        }

        return false;
    }

    public function getJobsQueue()
    {
        $result = array();
        $get_jobs = $this->db->query("select cmd_id,cmd,records,status_name,price_id 
                                                      from " . DB_PREFIX . "4dealer_cmd, " . DB_PREFIX . "4dealer_statuses 
                                                      where status_id in(1,4) 
                                                      and status=status_id ");
        if ($get_jobs->num_rows) {
            foreach ($get_jobs->rows as $job) {
                $result[$job['cmd']][$job['price_id']] = $job;
            }
        }

        return $result;

    }

    function updateDealerParams()
    {
        $result['res'] = 0;
        $price_id = isset($_REQUEST['dealer_id']) ? (int)$_REQUEST['dealer_id'] : false;

        if (!$price_id) {
            return $result;
        }

        $priceInfo = $this->getPriceInfoById($price_id);

        $data = array();

        $mapper = [
            'small_quantity' => 'small_quantity_transform',
            'medium_quantity' => 'medium_quantity_transform',
            'large_quantity' => 'large_quantity_transform',
            'price_markup' => 'price_markup',
            'round_price' => 'round_price',
            'round_rule' => 'round_rule',
            'product_add_table' => 'product_add_table',
            'other_images' => 'other_images',
            'meta' => 'meta'
        ];

        foreach ($mapper as $key => $value) {
            $data[$value] = $_REQUEST[$key];
        }

        if (!sizeof($data)) {
            return $result;
        }

        $data['meta'] = array_merge((array) $priceInfo['meta'], (array) $data['meta']);

        $data['meta'] = json_encode($data['meta']);

        $status = $this->db->query("UPDATE " . DB_PREFIX . "4dealer_prices 
        SET small_quantity_transform='{$data['small_quantity_transform']}', 
            medium_quantity_transform='{$data['medium_quantity_transform']}', large_quantity_transform='{$data['large_quantity_transform']}', 
            price_markup='{$data['price_markup']}', round_price='{$data['round_price']}', round_rule='{$data['round_rule']}',
            product_add_table='{$data['product_add_table']}', other_images='{$data['other_images']}', meta = '{$data['meta']}' 
        WHERE price_id='$price_id'");

        if ($status) {
            $result['res'] = 1;
        }

        return $result;
    }

    function deletePriceMarkup()
    {
        $result = ['res' => 0];
        $fields = ['id' => 'id'];

        foreach ($fields as $key => $value) {
            $delete_data[$value] = (int)$_REQUEST[$key];
        }

        $result['res'] = $this->db->query("delete from " . DB_PREFIX . "4dealer_price_markup where id={$delete_data['id']}");

        return $result;
    }

    function addPriceMarkup()
    {
        $result = ['res' => 0];

        $fields = [
            'dealer_id' => 'price_id',
            'category' => 'category_id',
            'price_from' => 'price_low',
            'price_to' => 'price_high',
            'price_markup' => 'markup',
            'price_is_rrc' => 'price_is_rrc',
        ];

        foreach ($fields as $key => $field) {
            if (!isset($_REQUEST[$key])) {
                return $result;
            }

            $data[$field] = (int)$_REQUEST[$key];
        }

        $res = $this->db->query(
            "INSERT INTO " . DB_PREFIX . "4dealer_price_markup 
                    (price_id,category_id,price_low,price_high,markup,price_is_rrc) 
             VALUES ({$data['price_id']}, {$data['category_id']}, {$data['price_low']}, {$data['price_high']},
                    {$data['markup']}, {$data['price_is_rrc']})"
        );

        if (!$res) {
            return $result;
        }

        $result['res'] = $res;
        $result['id'] = $this->db->getLastId();

        return $result;
    }

    function updatePriceMarkup()
    {
        $result = ['res' => 0];
        $fields = [
            'id' => 'id',
            'category' => 'category_id',
            'price_from' => 'price_low',
            'price_to' => 'price_high',
            'price_markup' => 'markup',
            'price_is_rrc' => 'price_is_rrc',
        ];

        foreach ($fields as $key => $field) {
            if (!isset($_REQUEST[$key]) || (empty($_REQUEST[$key]) && $_REQUEST[$key] != 0)) {
                return $result;
            }

            $data[$field] = (int)$_REQUEST[$key];
        }

        $this->db->query("UPDATE " . DB_PREFIX . "4dealer_price_markup 
        SET category_id = '{$data['category_id']}', price_low = '{$data['price_low']}', price_high = '{$data['price_high']}',
            markup  = '{$data['markup']}', price_is_rrc  = '{$data['price_is_rrc']}'
        WHERE id = {$data['id']}");

        return true;
    }

    function getPriceMarkupByPriceSectionId($priceId, $sectionId, $cache = false)
    {
        $cacheKey = 'getPriceMarkupByPriceSectionId:' . $priceId . ':' . $sectionId;

        if (isset($this->cacheSpace[$cacheKey]) && $cache) {
            return $this->cacheSpace[$cacheKey];
        }

        $sql = "SELECT * FROM `" . DB_PREFIX . "4dealer_price_markup` 
        WHERE price_id = '{$priceId}' AND (category_id IN(0, {$sectionId})) ORDER BY category_id DESC LIMIT 1";

        $result = $this->db->query($sql);

        return $this->cacheSpace[$cacheKey] = $result->row && isset($result->row['id']) ? $result->row : false;
    }

    function updatePriceMarkupAutoLoadOptions() {
        $price_id = $_REQUEST['dealer_id'];
        $autoLoadOptions = $_REQUEST['auto_load_options'];

        $this->db->query(
            "UPDATE " . DB_PREFIX . "4dealer_prices 
             SET auto_load_options = '{$autoLoadOptions}' WHERE price_id = {$price_id}"
        );

        return true;
    }

    function addOptionsMerge()
    {
        $result = ['res' => 0];

        $fields = [
            'dealer_id' => '4dealer_price_id',
            'category' => 'section_id',
            'option_name' => 'option_name',
            'changed_option_name' => 'changed_option_name'
        ];

        foreach ($fields as $key => $field) {
            if (!isset($_REQUEST[$key]) || (empty($_REQUEST[$key]) && $_REQUEST[$key] != 0)) {
                return $result;
            }

            $data[$field] = $_REQUEST[$key];
        }

        $result['res'] = $this->db->query(
            "INSERT INTO " . DB_PREFIX . "4dealer_options_merge (4dealer_price_id, section_id, option_name, changed_option_name) 
             VALUES ('{$data['4dealer_price_id']}', '{$data['section_id']}', '{$data['option_name']}', '{$data['changed_option_name']}')"
        );

        $result['id'] = $this->db->getLastId();

        return $result;
    }

    function multipleCreateOptions($price_id, $section_id)
    {
        $dealerCats = $this->getDealerCategoriesByLocalCatIds($price_id, [$section_id]);

        $options = ((array) $this->get_list_options_merge($dealerCats,false));

        $optionsList = array_column($options['Result'], 'option_name');
        $optionsList = array_filter(array_unique($optionsList));

        /** Получаем текущие свойства, исключаем дублирование */

        $existOptions = $this->getExistOptionsMergeBySectionId($price_id, $section_id);
        $existOptionNames = array_column($existOptions, 'option_name');

        $optionsList = array_diff($optionsList, $existOptionNames);
        $optionsList = array_unique($optionsList);
        $optionsList = array_filter($optionsList);

        $data = [
            '4dealer_price_id' => $price_id,
            'section_id' => $section_id,
            'option_name' => '',
            'changed_option_name' => '',
        ];

        foreach ($optionsList as $option) {
            $data['option_name'] = $option;

            $this->db->query(
                "INSERT INTO " . DB_PREFIX . "4dealer_options_merge (4dealer_price_id, section_id, option_name, changed_option_name) 
                 VALUES ('{$data['4dealer_price_id']}', '{$data['section_id']}', '{$data['option_name']}', '{$data['changed_option_name']}')"
            );
        }

        return true;
    }

    function getOptionsMergeByPriceId($price_id, $dealer_cat = null, $cache = false)
    {
        $cacheKey = 'getOptionsMergeByPriceId:' . $price_id . ':' . $dealer_cat;

        if ($cache && isset($this->cacheSpace[$cacheKey])) {
            return $this->cacheSpace[$cacheKey];
        }

        // Получаем локальную категорию для $dealer_cat
        $localCategories = $this->getLocalCatIdsByDealerCat($price_id, [$dealer_cat]);
        $localCategories = implode(',', $localCategories);

        // Получаем список всех аттрибутов по этой категории
        $sql = "SELECT * FROM " . DB_PREFIX . "4dealer_options_merge WHERE `4dealer_price_id` = {$price_id}";

        if (isset($localCategories) && !empty($localCategories))
            $sql .= " AND section_id IN ($localCategories)";

        return $this->cacheSpace[$cacheKey] = $this->db->query($sql)->rows;
    }

    function updateOptionsMerge()
    {
        $result = ['res' => 0];

        $fields = [
            'id' => 'id',
            'category' => 'section_id',
            'option_name' => 'option_name',
            'changed_option_name' => 'changed_option_name'
        ];

        $data = $this->getDataFields($fields);

        $result['res'] = $this->db->query(
            'UPDATE ' . DB_PREFIX . "4dealer_options_merge 
             SET section_id = '{$data['section_id']}', option_name = '{$data['option_name']}',
                 changed_option_name = '{$data['changed_option_name']}'
             WHERE id = '{$data['id']}'"
        );

        return $result;
    }

    function saveMergeReplaces()
    {
        $fields = [ 'id' => 'id', 'replaces' => 'replaces' ];

        $data = $this->getDataFields($fields);

        $data['replaces'] = json_encode((array) $data['replaces'], JSON_UNESCAPED_UNICODE);

        $this->db->query(
            "UPDATE " . DB_PREFIX . "4dealer_options_merge SET replaces = '{$data['replaces']}' 
             WHERE id = '{$data['id']}'"
        );

        return true;
    }

    function deleteOptionsMerge()
    {
        $id = $_REQUEST['id'];

        $this->db->query("DELETE FROM " . DB_PREFIX . "4dealer_options_merge WHERE id = '{$id}'");

        return true;
    }

    function deleteEmptyBindings()
    {
        $this->db->query("DELETE FROM " . DB_PREFIX . "4dealer_categories_term WHERE term_id NOT IN (
            SELECT category_id FROM " . DB_PREFIX . "category
        )");

        return true;
    }

    /**
     * Подсчитываем количество в существующих options_merge
     *
     * @return mixed
     */
    function countGroupOptionsMerge()
    {
        return $this->db->query(
            "SELECT section_id, COUNT(id) AS cnt FROM `".DB_PREFIX."4dealer_options_merge` GROUP BY section_id"
        );
    }

    /**
     * Фильтруем опции, заменяем названия и значения
     *
     * @param $priceInfo
     * @param $options
     * @param $optionsMerge
     * @return mixed
     */
    function filterOptions($priceInfo, $options, $optionsMerge)
    {
        $result = [];

        if ($priceInfo['auto_load_options'] == '0') {

            /** Пересекаем множество созданных свойств ($optionsMerge) с переданными по Api ($options) */

            $options = (array) $options;

            foreach ($options as $option) {
                foreach ($optionsMerge as $optionMerge) {
                    $optionMerge = (array) $optionMerge;

                    if (trim($option->name) == trim($optionMerge['option_name'])) {
                        $result[] = $option;
                    }
                }
            }

        } else {
            $result = $options;
        }

        /** Заменяем названия и значения для оставшихся (если были отфильтрованы) опций */

        foreach ($result as $option) {
            $this->attributeMergeReplaces($option, $optionsMerge);
        }

        return $result;
    }

    function attributeMergeReplaces(&$attribute, $optionsMerge)
    {
        foreach ($optionsMerge as $option) {

            if ($option['option_name'] == $attribute->name || $option['changed_option_name'] == $attribute->name) {

                if ($option['option_name'] == $attribute->name && !empty($option['changed_option_name'])) {
                    $attribute->name = $option['changed_option_name'];
                }

                if (!empty($option['replaces'])) {
                    $option['replaces'] = json_decode($option['replaces'], true);

                    foreach ($option['replaces'] as $find => $replace) {
                        $attribute->value = str_replace($find, $replace, $attribute->value);
                    }
                }

            }
        }
    }

    /**
     * Получение цены для продукта из ::createProducts
     *
     * @param $priceInfo - информация о прайсе
     * @param $product - информация о продукте из импорта (createProduct)
     * @param $price
     * @return float
     */
    function checkoutPrice($priceInfo, $product, $price)
    {
        /** Получение цены - включена ли msrp_r опция в наценках */
        $markup = $this->getPriceMarkupByPriceSectionId($product['price_id'], $product['price_category_id'], true);

        $price = ($markup && $markup['price_is_rrc'] == 1 && $product['msrp_r']) ? $product['msrp_r'] : $price;

        /** Округление включено в настройках */
        if ($priceInfo['round_price'] != 0) {
            $this->logger('Округление включено в настройках, округляем цену');

            $roundPrice = $priceInfo['round_price'] ? $priceInfo['round_price'] : false;
            $roundRule = $priceInfo['round_rule'] == 0 ? PHP_ROUND_HALF_UP : PHP_ROUND_HALF_DOWN;

            // У округления до целого значение в select = 1, меняем на 0
            if ($roundPrice == 1 || !$roundPrice) {
                $roundPrice = 0;
                $this->logger('...до целого');
            }

            if (round($price, $roundPrice, $roundRule) > 0) {
                $price = round($price, $roundPrice, $roundRule);
            } else {
                $price = round($price, 0, $roundRule);
            }
        }

        return $price;
    }

    /**
     * Получение всех аттрибутов по локальной категории
     *
     * @param $price_id
     * @param $section_id
     * @return array
     */
    function getExistOptionsMergeBySectionId($price_id, $section_id)
    {
        $existOptions = $this->db->query(
            "SELECT * FROM " . DB_PREFIX . "4dealer_options_merge WHERE section_id = '{$section_id}' AND 4dealer_price_id = '{$price_id}'"
        );

        return (array) $existOptions->rows;
    }

    function addJob($cmd_list = false, $additional_data = null, $ajax = true, $parent = 0, $priority = 50)
    {
        $result = array('res' => 0);

        $priorities = ['short_price_update' => 10, 'full_price_update' => 11];

        if ($cmd_list && is_array($cmd_list)) {
            foreach ($cmd_list as $cmd => $data) {
                foreach ($data as $price_id) {
                    if (isset($_REQUEST['cmd_list']) && isset($priorities[$cmd])) {
                        $priority = $priorities[$cmd];
                    }

                    $this->db->query("insert into " . DB_PREFIX . "4dealer_cmd(cmd,parent_id,create_time,price_id,data,priority) 
                                        values('$cmd',$parent,now(),$price_id,'" . $this->db->escape(serialize($additional_data)) . "',$priority)");
                    $result['data'][$cmd][$price_id] = $this->db->getLastId();
                }
            }
            if (isset($result['data'])) {
                $result['res'] = 1;
            }
        }

        if ($ajax) {
            return $result;
        }

        return true;
    }

    function job_name($cmd) {
        switch ($cmd) {
            case 'short_price_update':
                return 'Быстрое обновление';
            case 'full_price_update':
                return 'Полное обновление';
            case 'clear_old_products':
                return 'Чистка задач';
            case 'update_products':
                return 'Обновление продуктов';
            case 'create_products':
                return 'Создание продуктов';
            case 'hide_duplicates':
                return 'Скрытие дубликатов';
            case 'get_products_info':
                return 'Создание карточек продуктов';

            default:
                return $cmd;
        }
    }

    function job_status($status)
    {
        switch ($status) {
            case '1':
                return 'Запущен';
            case '2':
                return 'Готово';
            case '3':
                return 'Ошибка';
            case '4':
                return 'В очереди';
            case '5':
                return 'Отменен';

            default:
                return $status;
        }
    }

    function getJobsStatusByArrayId()
    {
        $result = array('res' => 0);
        if (isset($_REQUEST['cmd_list']) && is_array($_REQUEST['cmd_list'])) {
            foreach ($_REQUEST['cmd_list'] as $cmd_id) {
                $cmd_id_list[] = (int)$cmd_id;
            }
            $get_jobs = $this->db->query("select cmd_id,cmd,records,status_name 
                                    from " . DB_PREFIX . "4dealer_cmd, " . DB_PREFIX . "4dealer_statuses 
                                    where cmd_id in(" . implode(',', $cmd_id_list) . ") and status=status_id");
            if (!$get_jobs->num_rows) {
                return $result;
            }
            foreach ($get_jobs->rows as $job) {
                $result['data'][$job['cmd_id']] = $job;
                if ($job['cmd'] == 'full_price_update' || $job['cmd'] == 'short_price_update') {

                    $total_records = $this->db->query("select sum(records) as cnt  
                                                      from " . DB_PREFIX . "4dealer_cmd  
                                                      where parent_id = {$job['cmd_id']} and cmd in ('createProducts', 'updateProducts')");

                    $count_wait_and_run_jobs = $this->db->query("select count(cmd_id) as cnt  
                                                      from " . DB_PREFIX . "4dealer_cmd  
                                                      where parent_id = {$job['cmd_id']} and status in (1,4) and cmd in ('createProducts', 'updateProducts')");


                    $result['data'][$job['cmd_id']]['records'] = is_null($total_records->row['cnt']) ? 0 : $total_records->row['cnt'];
                    $result['data'][$job['cmd_id']]['jobs'] = $count_wait_and_run_jobs->row['cnt'];

                    if ($count_wait_and_run_jobs->row['cnt'] == 0 && !($job['status_name'] == 'Running' || $job['status_name'] == 'Waiting')) {
                        $result['data'][$job['cmd_id']]['status_name'] = 'Finished';
                    } else {
                        if ($job['status_name'] == 'Finished') {
                            $result['data'][$job['cmd_id']]['status_name'] = 'Running';
                        }
                    }

                }


            }
            if (isset($result['data'])) {
                $result['res'] = 1;
            }
        }
        return $result;
    }

    function deleteOldJobs()
    {
        $this->db->query("DELETE FROM " . DB_PREFIX . "4dealer_cmd where start_time < now() - interval 48 hour and status not in(1,4)");
    }

    function cancelCommand($cmd_id = false, $ajax = true)
    {
        $result['res'] = 0;

        if ($cmd_id) {
            $data = array('status' => 5);
            if ($this->updateJobById($cmd_id, $data) && $this->updateJobsByParentId($cmd_id, $data)) {
                $result['res'] = 1;
            }

        }
        if ($ajax) {
            return $result;
        }
    }

    function updateJobById($job_id, $data)
    {
        $upd_data = [];
        $numeric_fields = ['status', 'records', 'pid', 'priority'];

        foreach ($data as $key => $value) {
            if (in_array($key, $numeric_fields)) {
                $upd_data[] = "$key=" . (int)$value;

            } else {
                $value = $this->db->escape($value);
                $upd_data[] = "$key='$value'";
            }
        }

        $this->db->query(
            "UPDATE " . DB_PREFIX . "4dealer_cmd SET " . implode(',', $upd_data) . " WHERE cmd_id = {$job_id}"
        );

        return true;
    }

    function deleteJobById($job_id = null)
    {
        $this->db->query("DELETE FROM " . DB_PREFIX . "4dealer_cmd " . ($job_id ? "WHERE cmd_id={$job_id}" : ''));

        return true;
    }

    function deleteFinishedJobs()
    {
        return $this->db->query("DELETE FROM " . DB_PREFIX . "4dealer_cmd WHERE status = 2");
    }

    function updateJobsByParentId($parent_id, $data)
    {
        $this->db->query("update " . DB_PREFIX . "4dealer_cmd set status='" . (int)$data['status'] . "' where parent_id=" . (int)$parent_id);
        return true;
    }

    function cancelDeadCommands()
    {
        $get_jobs = $this->db->query("select cmd_id,pid 
                                      from " . DB_PREFIX . "4dealer_cmd, " . DB_PREFIX . "4dealer_statuses 
                                      where status_id=1 and status=status_id");

        if (!$get_jobs->num_rows) {
            return false;
        }
        foreach ($get_jobs->rows as $job) {
            if (!$this->checkPid($job['pid'])) {
                $this->logger("[{$job['cmd_id']}] Завершаем зависший процесс!");
                $this->cancelCommand($job['cmd_id'], false);
            }
        }
    }

    function checkPid($pid)
    {
        $file_path = "/proc/$pid/cmdline";
        if (is_file($file_path)) {
            $res = file_get_contents($file_path);
            if ($res) {
                return strpos($res, 'oc_cli.php');
            } else {
                return true;
            }
        }
        return false;
    }

    function getJobStatusById($id)
    {
        return $this->db->query("select status from " . DB_PREFIX . "4dealer_cmd where cmd_id=" . (int)$id)->row['status'];
    }

    function getProcessJobsList($page = 1)
    {
        $jobs = [];

        $pageSize = 25;
        $jobsCount = $this->db->query("SELECT COUNT(*) AS num FROM " . DB_PREFIX . "4dealer_cmd")->row['num'];

        $totalPages = ceil($jobsCount / $pageSize);

        $offset = ($page * $pageSize) - $pageSize;

        $jobsDb = $this->db->query(
            "SELECT * FROM " . DB_PREFIX . "4dealer_cmd ORDER BY status, end_time DESC, create_time DESC LIMIT {$offset}, {$pageSize}"
        );

        foreach ($jobsDb->rows as $job) {
            $job['name'] = $this->job_name($job['cmd']);
            $job['named_status'] = $this->job_status($job['status']);

            $jobs[$job['cmd_id']] = $job;
        }

        $pricesDb = (array) $this->QB->table(DB_PREFIX . '4dealer_prices')->get();

        foreach ($pricesDb as $price) {
            foreach ($jobs as &$job) {
                if ($job['price_id'] == $price['price_id']) {
                    $job['price_name'] = $price['price_name'];
                }
            }
        }

        return [
            'jobs' => $jobs,
            'page' => $page,
            'pages' => $totalPages,
        ];
    }

    function processKiller($job_id, $current_records)
    {
        $current_status = $this->getJobStatusById($job_id);

        if ($current_status == 5) {
            $data = array('records' => $current_records, 'end_time' => $this->getCurrentMysqlTime());
            $this->updateJobById($job_id, $data);

            exit();
        }
    }

    function getCurrentMysqlTime()
    {
        $get_time = $this->db->query("select now() as now");

        return $get_time->row['now'];
    }

    function setInactiveCategories($id_list, $price_id)
    {
        $this->db->query("update " . DB_PREFIX . "4dealer_categories_term set is_active=1 where 4dealer_price_id=$price_id");
        if (sizeof($id_list)) {
            $this->db->query("update " . DB_PREFIX . "4dealer_categories_term set is_active=0 where 4dealer_price_id=$price_id and 4dealer_cat_id not in(" . implode(',', $id_list) . ")");
        }
    }

    function getProductByForDealerId($id)
    {
        $get_product = $this->db->query("select * from " . DB_PREFIX . "product where dealer_id=$id");
        if ($get_product->num_rows) {
            return $get_product->row;
        }
        return false;
    }

    function getProductOldDescription($id, $language_id = 1)
    {
        if (!$id) {
            return false;
        }

        $sql = "select * from " . DB_PREFIX . "product_description where product_id={$id} AND language_id={$language_id}";
        $get_product = $this->db->query($sql);

        if ($get_product->num_rows) {
            return $get_product->row;
        }

        return false;
    }

    function getManufacturerByName($name)
    {
        $name = $this->db->escape($name);
        $get_manufacturer = $this->db->query("select * from " . DB_PREFIX . "manufacturer where name='$name'");
        if ($get_manufacturer->num_rows) {
            return $get_manufacturer->row;
        }
        return false;
    }

    function getQuantity($quantity, $small_quantity_transform, $medium_quantity_transform, $large_quantity_transform)
    {
        if (in_array(strtolower($quantity), array('', 'false', 'call', 'no', ' '))) {
            $newQuantity = 0;
        } else {
            switch ($quantity) {
                case '*':
                    $newQuantity = $small_quantity_transform;
                    break;
                case '**':
                    $newQuantity = $medium_quantity_transform;
                    break;
                case '***':
                    $newQuantity = $large_quantity_transform;
                    break;
                case 'Да':
                    $newQuantity = $large_quantity_transform;
                    break;
                default :
                    $newQuantity = preg_replace('~[^0-9]+~', '', $quantity);
                    break;
            }
        }

        if (empty($newQuantity))
            $newQuantity = 0;

        return $newQuantity;


    }

    function isPriceInRange($markup, $price)
    {
        return $markup['price_low'] <= (float)$price && (float)$price <= $markup['price_high'];
    }

    function calculateMarkup($markup, $price)
    {
        return (float)$price + round((float)$price * (float)$markup / 100, 2);
    }

    function iterateMarkups($markups, $price)
    {
        foreach ($markups as $markup) {
            if ($this->isPriceInRange($markup, $price)) {
                $this->logger('Учитываем наценку: ' . $markup['markup'] . '% с учетом цены ' . $price);
                return $this->calculateMarkup($markup['markup'], $price);

            }
        }
        return false;
    }

    function recalculatePrice($general_markup, $categories_markup, $category_id, $price)
    {

        $newPrice = $price;

        if ($general_markup || $categories_markup) {
            if (isset($categories_markup[$category_id])) {
                if ($newPrice = $this->iterateMarkups($categories_markup[$category_id], $price)) {
                    return $newPrice;
                }
            } elseif (isset($categories_markup[0])) {
                if ($newPrice = $this->iterateMarkups($categories_markup[0], $price)) {
                    return $newPrice;
                }
            }

            $newPrice = $this->calculateMarkup($general_markup, $price);
        }

        return $newPrice;
    }

    public function preSaveProduct(&$data)
    {
        if (isset($data['model'])) {
            $data['model'] = str_replace('"', '&quot;', $data['model']);
        }

        if (isset($data['product_description'])) {
            foreach ($data['product_description'] as $language_id => &$value) {
                $value['name'] = str_replace('"', '&quot;', $value['name']);
            }

        }
    }

    public function addProductWithDealerData($data)
    {
        $this->preSaveProduct($data);
        
        $timestamp = $this->getCurrentMysqlTime();
        $this->db->query(
            "INSERT INTO " . DB_PREFIX . "product SET DuID={$data['DuID']}, dealer_id={$data['dealer_id']}, dealer_price_id={$data['dealer_price_id']}, 
             model = '" . $this->db->escape($data['model']) . "', sku = '" . $this->db->escape($data['sku']) . "', upc = '" . $this->db->escape($data['upc']) . "',
             ean = '" . $this->db->escape($data['ean']) . "', jan = '" . $this->db->escape($data['jan']) . "', isbn = '" . $this->db->escape($data['isbn']) . "',
             mpn = '" . $this->db->escape($data['mpn']) . "', location = '" . $this->db->escape($data['location']) . "', quantity = '" . (int)$data['quantity'] . "',
             minimum = '" . (int)$data['minimum'] . "', subtract = '" . (int)$data['subtract'] . "', stock_status_id = '" . (int)$data['stock_status_id'] . "',
             date_available = '" . $this->db->escape($data['date_available']) . "', manufacturer_id = '" . (int)$data['manufacturer_id'] . "',
             shipping = '" . (int)$data['shipping'] . "', price = '" . (float)$data['price'] . "', points = '" . (int)$data['points'] . "',
             weight = '" . (float)$data['weight'] . "', weight_class_id = '" . (int)$data['weight_class_id'] . "', length = '" . (float)$data['length'] . "',
             width = '" . (float)$data['width'] . "', height = '" . (float)$data['height'] . "', length_class_id = '" . (int)$data['length_class_id'] . "',
             status = '" . (int)$data['status'] . "', tax_class_id = '" . (int)$data['tax_class_id'] . "', sort_order = '" . (int)$data['sort_order'] . "',
             date_added = '" . $timestamp . "', date_modified = '" . $timestamp . "'"
        );
        $product_id = $this->db->getLastId();

        if (isset($data['image'])) {
            $this->db->query("UPDATE " . DB_PREFIX . "product SET image = '" . $this->db->escape($data['image']) . "' WHERE product_id = '" . (int)$product_id . "'");
        }

        foreach ($data['product_description'] as $language_id => $value) {
            $this->db->query(
                "INSERT INTO " . DB_PREFIX . "product_description SET product_id = '" . (int)$product_id
                . "', language_id = '" . (int)$language_id . "', name = '" . $this->db->escape($value['name'])
                . "', description = '" . $this->db->escape($value['description']) . "', tag = '" . $this->db->escape($value['tag'])
                . "', meta_title = '" . $this->db->escape($value['meta_title']) . "', meta_description = '" . $this->db->escape($value['meta_description'])
                . "', meta_keyword = '" . $this->db->escape($value['meta_keyword']) . "'"
            );
        }

        if (isset($data['product_store'])) {
            foreach ($data['product_store'] as $store_id) {
                $this->db->query("INSERT INTO " . DB_PREFIX . "product_to_store SET product_id = '" . (int)$product_id . "', store_id = '" . (int)$store_id . "'");
            }
        }

        if (isset($data['product_attribute'])) {
            foreach ($data['product_attribute'] as $product_attribute) {
                if ($product_attribute['attribute_id']) {
                    // Removes duplicates
                    $this->db->query("DELETE FROM " . DB_PREFIX . "product_attribute WHERE product_id = '" . (int)$product_id . "' AND attribute_id = '" . (int)$product_attribute['attribute_id'] . "'");

                    foreach ($product_attribute['product_attribute_description'] as $language_id => $product_attribute_description) {
                        $this->db->query("DELETE FROM " . DB_PREFIX . "product_attribute WHERE product_id = '" . (int)$product_id . "' AND attribute_id = '" . (int)$product_attribute['attribute_id'] . "' AND language_id = '" . (int)$language_id . "'");

                        $this->db->query("INSERT INTO " . DB_PREFIX . "product_attribute SET product_id = '" . (int)$product_id . "', attribute_id = '" . (int)$product_attribute['attribute_id'] . "', language_id = '" . (int)$language_id . "', text = '" . $this->db->escape($product_attribute_description['text']) . "'");
                    }
                }
            }
        }

        if (isset($data['product_option'])) {
            foreach ($data['product_option'] as $product_option) {

                if ($product_option['type'] == 'select' || $product_option['type'] == 'radio' || $product_option['type'] == 'checkbox' || $product_option['type'] == 'image') {
                    if (isset($product_option['product_option_value'])) {

                        $this->db->query(
                            "INSERT INTO " . DB_PREFIX . "product_option SET product_id = '" . (int)$product_id . "', 
                             option_id = '" . (int)$product_option['option_id'] . "', required = '" . (int)$product_option['required'] . "'"
                        );

                        $product_option_id = $this->db->getLastId();

                        foreach ($product_option['product_option_value'] as $product_option_value) {
                            $this->db->query(
                                "INSERT INTO " . DB_PREFIX . "product_option_value SET
                                 product_option_id = '" . (int)$product_option_id . "',
                                 product_id = '" . (int)$product_id . "',
                                 option_id = '" . (int)$product_option['option_id'] . "',
                                 option_value_id = '" . (int)$product_option_value['option_value_id'] . "',
                                 quantity = '" . (int)$product_option_value['quantity'] . "',
                                 subtract = '" . (int)$product_option_value['subtract'] . "',
                                 price = '" . (float)$product_option_value['price'] . "',
                                 price_prefix = '" . $this->db->escape($product_option_value['price_prefix']) . "',
                                 points = '" . (int)$product_option_value['points'] . "',
                                 points_prefix = '" . $this->db->escape($product_option_value['points_prefix']) . "',
                                 weight = '" . (float)$product_option_value['weight'] . "',
                                 weight_prefix = '" . $this->db->escape($product_option_value['weight_prefix']) . "'"
                            );
                        }
                    }

                } else {
                    $this->db->query(
                        "INSERT INTO " . DB_PREFIX . "product_option SET product_id = '" . (int)$product_id . "',
                         option_id = '" . (int)$product_option['option_id'] . "', value = '" . $this->db->escape($product_option['value']) . "',
                         required = '" . (int)$product_option['required'] . "'"
                    );
                }

            }
        }

        if (isset($data['product_recurring'])) {
            foreach ($data['product_recurring'] as $recurring) {
                $this->db->query("INSERT INTO `" . DB_PREFIX . "product_recurring` SET `product_id` = " . (int)$product_id . ", customer_group_id = " . (int)$recurring['customer_group_id'] . ", `recurring_id` = " . (int)$recurring['recurring_id']);
            }
        }

        if (isset($data['product_discount'])) {
            foreach ($data['product_discount'] as $product_discount) {
                $this->db->query("INSERT INTO " . DB_PREFIX . "product_discount SET product_id = '" . (int)$product_id . "', customer_group_id = '" . (int)$product_discount['customer_group_id'] . "', quantity = '" . (int)$product_discount['quantity'] . "', priority = '" . (int)$product_discount['priority'] . "', price = '" . (float)$product_discount['price'] . "', date_start = '" . $this->db->escape($product_discount['date_start']) . "', date_end = '" . $this->db->escape($product_discount['date_end']) . "'");
            }
        }

        if (isset($data['product_special'])) {
            foreach ($data['product_special'] as $product_special) {
                $this->db->query("INSERT INTO " . DB_PREFIX . "product_special SET product_id = '" . (int)$product_id . "', customer_group_id = '" . (int)$product_special['customer_group_id'] . "', priority = '" . (int)$product_special['priority'] . "', price = '" . (float)$product_special['price'] . "', date_start = '" . $this->db->escape($product_special['date_start']) . "', date_end = '" . $this->db->escape($product_special['date_end']) . "'");
            }
        }

        if (isset($data['product_image'])) {
            foreach ($data['product_image'] as $product_image) {
                $this->db->query("INSERT INTO " . DB_PREFIX . "product_image SET product_id = '" . (int)$product_id . "', image = '" . $this->db->escape($product_image['image']) . "', sort_order = '" . (int)$product_image['sort_order'] . "'");
            }
        }

        if (isset($data['product_download'])) {
            foreach ($data['product_download'] as $download_id) {
                $this->db->query("INSERT INTO " . DB_PREFIX . "product_to_download SET product_id = '" . (int)$product_id . "', download_id = '" . (int)$download_id . "'");
            }
        }

        if (isset($data['product_category'])) {
            foreach ($data['product_category'] as $category_id) {
                $this->db->query("INSERT INTO " . DB_PREFIX . "product_to_category SET product_id = '" . (int)$product_id . "', category_id = '" . (int)$category_id . "'");
            }
        }

        if (isset($data['product_filter'])) {
            foreach ($data['product_filter'] as $filter_id) {
                $this->db->query("INSERT INTO " . DB_PREFIX . "product_filter SET product_id = '" . (int)$product_id . "', filter_id = '" . (int)$filter_id . "'");
            }
        }

        if (isset($data['product_related'])) {
            foreach ($data['product_related'] as $related_id) {
                $this->db->query("DELETE FROM " . DB_PREFIX . "product_related WHERE product_id = '" . (int)$product_id . "' AND related_id = '" . (int)$related_id . "'");
                $this->db->query("INSERT INTO " . DB_PREFIX . "product_related SET product_id = '" . (int)$product_id . "', related_id = '" . (int)$related_id . "'");
                $this->db->query("DELETE FROM " . DB_PREFIX . "product_related WHERE product_id = '" . (int)$related_id . "' AND related_id = '" . (int)$product_id . "'");
                $this->db->query("INSERT INTO " . DB_PREFIX . "product_related SET product_id = '" . (int)$related_id . "', related_id = '" . (int)$product_id . "'");
            }
        }

        if (isset($data['product_reward'])) {
            foreach ($data['product_reward'] as $customer_group_id => $product_reward) {
                if ((int)$product_reward['points'] > 0) {
                    $this->db->query("INSERT INTO " . DB_PREFIX . "product_reward SET product_id = '" . (int)$product_id . "', customer_group_id = '" . (int)$customer_group_id . "', points = '" . (int)$product_reward['points'] . "'");
                }
            }
        }

        // SEO URL
        if (isset($data['product_seo_url'])) {
            foreach ($data['product_seo_url'] as $store_id => $language) {
                foreach ($language as $language_id => $keyword) {
                    if (!empty($keyword)) {
                        $this->db->query("INSERT INTO " . DB_PREFIX . "seo_url SET store_id = '" . (int)$store_id . "', language_id = '" . (int)$language_id . "', query = 'product_id=" . (int)$product_id . "', keyword = '" . $this->db->escape($keyword) . "'");
                    }
                }
            }
        }

        if (isset($data['product_layout'])) {
            foreach ($data['product_layout'] as $store_id => $layout_id) {
                $this->db->query("INSERT INTO " . DB_PREFIX . "product_to_layout SET product_id = '" . (int)$product_id . "', store_id = '" . (int)$store_id . "', layout_id = '" . (int)$layout_id . "'");
            }
        }


        $this->cache->delete('product');

        return $product_id;
    }

    public function editProductWithDealerData($product_id, $data)
    {
        if (!$product_id) {
            $this->logger('editProductWithDealerData - товар не найден');
            return false;
        }

        $this->preSaveProduct($data);

        $this->db->query("
            UPDATE " . DB_PREFIX . "product SET DuID='{$data['DuID']}', dealer_id=" . (int)$data['dealer_id'] . ", dealer_price_id=" . (int)$data['dealer_price_id'] . ",
            model = '" . $this->db->escape($data['model']) . "', sku = '" . $this->db->escape($data['sku']) . "', upc = '" . $this->db->escape($data['upc']) . "',
            ean = '" . $this->db->escape($data['ean']) . "', jan = '" . $this->db->escape($data['jan']) . "', isbn = '" . $this->db->escape($data['isbn']) . "',
            mpn = '" . $this->db->escape($data['mpn']) . "', location = '" . $this->db->escape($data['location']) . "', quantity = '" . (int)$data['quantity'] . "',
            minimum = '" . (int)$data['minimum'] . "', subtract = '" . (int)$data['subtract'] . "', stock_status_id = '" . (int)$data['stock_status_id'] . "',
            date_available = '" . $this->db->escape($data['date_available']) . "', manufacturer_id = '" . (int)$data['manufacturer_id'] . "',
            shipping = '" . (int)$data['shipping'] . "', price = '" . (float)$data['price'] . "', points = '" . (int)$data['points'] . "',
            weight = '" . (float)$data['weight'] . "', weight_class_id = '" . (int)$data['weight_class_id'] . "',
            length = '" . (float)$data['length'] . "', width = '" . (float)$data['width'] . "', height = '" . (float)$data['height'] . "',
            length_class_id = '" . (int)$data['length_class_id'] . "', status = '" . (int)$data['status'] . "',
            tax_class_id = '" . (int)$data['tax_class_id'] . "', sort_order = '" . (int)$data['sort_order'] . "',
            date_modified = NOW() WHERE product_id = '" . (int)$product_id . "'"
        );

        if (isset($data['image'])) {
            $this->db->query("UPDATE " . DB_PREFIX . "product SET image = '" . $this->db->escape($data['image']) . "' WHERE product_id = '" . (int)$product_id . "'");
        }


        if (isset($data['product_description'])) {
            $this->db->query("DELETE FROM " . DB_PREFIX . "product_description WHERE product_id = '" . (int)$product_id . "'");

            foreach ($data['product_description'] as $language_id => $value) {
                if (!isset($value['tag'])) continue;

                $this->db->query(
                    "INSERT INTO " . DB_PREFIX . "product_description SET product_id = '" . (int)$product_id . "', 
                     language_id = '" . (int)$language_id . "', name = '" . $this->db->escape($value['name']) . "', 
                     description = '" . $this->db->escape($value['description']) . "', tag = '" . $this->db->escape($value['tag']) . "', 
                     meta_title = '" . $this->db->escape($value['meta_title']) . "', meta_description = '" . $this->db->escape($value['meta_description']) . "', 
                     meta_keyword = '" . $this->db->escape($value['meta_keyword']) . "'"
                );
            }
        }


        if (isset($data['product_store'])) {
            $this->db->query("DELETE FROM " . DB_PREFIX . "product_to_store WHERE product_id = '" . (int)$product_id . "'");

            foreach ($data['product_store'] as $store_id) {
                $this->db->query("INSERT INTO " . DB_PREFIX . "product_to_store SET product_id = '" . (int)$product_id . "', store_id = '" . (int)$store_id . "'");
            }
        }


        if (!empty($data['product_attribute'])) {
            foreach ($data['product_attribute'] as $product_attribute) {
                if ($product_attribute['attribute_id']) {
                    // Removes duplicates
                    $this->db->query(
                        "DELETE FROM " . DB_PREFIX . "product_attribute 
                         WHERE product_id = '" . (int)$product_id . "' AND attribute_id = '" . (int)$product_attribute['attribute_id'] . "'"
                    );

                    foreach ($product_attribute['product_attribute_description'] as $language_id => $product_attribute_description) {
                        $this->db->query(
                            "INSERT INTO " . DB_PREFIX . "product_attribute SET product_id = '" . (int)$product_id . "',
                             attribute_id = '" . (int)$product_attribute['attribute_id'] . "', language_id = '" . (int)$language_id . "',
                             text = '" . $this->db->escape($product_attribute_description['text']) . "'"
                        );
                    }
                }
            }
        }

        if (isset($data['product_option'])) {

            $this->db->query("DELETE FROM " . DB_PREFIX . "product_option WHERE product_id = '" . (int)$product_id . "'");
            $this->db->query("DELETE FROM " . DB_PREFIX . "product_option_value WHERE product_id = '" . (int)$product_id . "'");

            foreach ($data['product_option'] as $product_option) {
                if ($product_option['type'] == 'select' || $product_option['type'] == 'radio' || $product_option['type'] == 'checkbox' || $product_option['type'] == 'image') {
                    if (isset($product_option['product_option_value'])) {

                        $this->db->query(
                            "INSERT INTO " . DB_PREFIX . "product_option SET product_option_id = '" . (int)$product_option['product_option_id'] . "',
                             product_id = '" . (int)$product_id . "', option_id = '" . (int)$product_option['option_id'] . "', 
                             required = '" . (int)$product_option['required'] . "'"
                        );

                        $product_option_id = $this->db->getLastId();

                        foreach ($product_option['product_option_value'] as $product_option_value) {
                            $this->db->query(
                                "INSERT INTO " . DB_PREFIX . "product_option_value SET 
                                 product_option_value_id = '" . (int)$product_option_value['product_option_value_id'] . "',
                                 product_option_id = '" . (int)$product_option_id . "',
                                 product_id = '" . (int)$product_id . "',
                                 option_id = '" . (int)$product_option['option_id'] . "',
                                 option_value_id = '" . (int)$product_option_value['option_value_id'] . "',
                                 quantity = '" . (int)$product_option_value['quantity'] . "',
                                 subtract = '" . (int)$product_option_value['subtract'] . "',
                                 price = '" . (float)$product_option_value['price'] . "',
                                 price_prefix = '" . $this->db->escape($product_option_value['price_prefix']) . "',
                                 points = '" . (int)$product_option_value['points'] . "',
                                 points_prefix = '" . $this->db->escape($product_option_value['points_prefix']) . "',
                                 weight = '" . (float)$product_option_value['weight'] . "',
                                 weight_prefix = '" . $this->db->escape($product_option_value['weight_prefix']) . "'"
                            );
                        }
                    }
                } else {
                    $this->db->query(
                        "INSERT INTO " . DB_PREFIX . "product_option SET product_option_id = '" . (int)$product_option['product_option_id'] . "',
                         product_id = '" . (int)$product_id . "', option_id = '" . (int)$product_option['option_id'] . "', 
                         value = '" . $this->db->escape($product_option['value']) . "', 
                         required = '" . (int)$product_option['required'] . "'"
                    );
                }
            }
        }


        if (isset($data['product_recurring'])) {
            $this->db->query("DELETE FROM `" . DB_PREFIX . "product_recurring` WHERE product_id = " . (int)$product_id);

            foreach ($data['product_recurring'] as $product_recurring) {
                $this->db->query(
                    "INSERT INTO `" . DB_PREFIX . "product_recurring` SET `product_id` = " . (int)$product_id . ",
                    customer_group_id = " . (int)$product_recurring['customer_group_id'] . ", `recurring_id` = " . (int)$product_recurring['recurring_id']
                );
            }
        }

        $this->db->query("DELETE FROM " . DB_PREFIX . "product_discount WHERE product_id = '" . (int)$product_id . "'");

        if (isset($data['product_discount'])) {
            foreach ($data['product_discount'] as $product_discount) {
                $this->db->query(
                    "INSERT INTO " . DB_PREFIX . "product_discount SET product_id = '" . (int)$product_id . "',
                     customer_group_id = '" . (int)$product_discount['customer_group_id'] . "',
                     quantity = '" . (int)$product_discount['quantity'] . "', priority = '" . (int)$product_discount['priority'] . "',
                     price = '" . (float)$product_discount['price'] . "', date_start = '" . $this->db->escape($product_discount['date_start']) . "',
                     date_end = '" . $this->db->escape($product_discount['date_end']) . "'"
                );
            }
        }


        if (isset($data['product_special'])) {
            $this->db->query("DELETE FROM " . DB_PREFIX . "product_special WHERE product_id = '" . (int)$product_id . "'");

            foreach ($data['product_special'] as $product_special) {
                $this->db->query(
                    "INSERT INTO " . DB_PREFIX . "product_special SET product_id = '" . (int)$product_id . "', 
                     customer_group_id = '" . (int)$product_special['customer_group_id'] . "', priority = '" . (int)$product_special['priority'] . "',
                     price = '" . (float)$product_special['price'] . "', date_start = '" . $this->db->escape($product_special['date_start']) . "', 
                     date_end = '" . $this->db->escape($product_special['date_end']) . "'"
                );
            }
        }


        if (isset($data['product_image'])) {
            $this->db->query("DELETE FROM " . DB_PREFIX . "product_image WHERE product_id = '" . (int)$product_id . "'");

            foreach ($data['product_image'] as $product_image) {
                $this->db->query(
                    "INSERT INTO " . DB_PREFIX . "product_image SET product_id = '" . (int)$product_id . "', 
                     image = '" . $this->db->escape($product_image['image']) . "', 
                     sort_order = '" . (int)$product_image['sort_order'] . "'"
                );
            }
        }


        if (isset($data['product_download'])) {
            $this->db->query("DELETE FROM " . DB_PREFIX . "product_to_download WHERE product_id = '" . (int)$product_id . "'");

            foreach ($data['product_download'] as $download_id) {
                $this->db->query("INSERT INTO " . DB_PREFIX . "product_to_download SET product_id = '" . (int)$product_id . "', download_id = '" . (int)$download_id . "'");
            }
        }


        if (isset($data['product_category'])) {
            $this->db->query("DELETE FROM " . DB_PREFIX . "product_to_category WHERE product_id = '" . (int)$product_id . "'");

            foreach ($data['product_category'] as $category_id) {
                $this->db->query("INSERT INTO " . DB_PREFIX . "product_to_category SET product_id = '" . (int)$product_id . "', category_id = '" . (int)$category_id . "'");
            }
        }


        if (isset($data['product_filter'])) {
            $this->db->query("DELETE FROM " . DB_PREFIX . "product_filter WHERE product_id = '" . (int)$product_id . "'");
            foreach ($data['product_filter'] as $filter_id) {
                $this->db->query("INSERT INTO " . DB_PREFIX . "product_filter SET product_id = '" . (int)$product_id . "', filter_id = '" . (int)$filter_id . "'");
            }
        }


        if (isset($data['product_related'])) {
            $this->db->query("DELETE FROM " . DB_PREFIX . "product_related WHERE product_id = '" . (int)$product_id . "'");
            $this->db->query("DELETE FROM " . DB_PREFIX . "product_related WHERE related_id = '" . (int)$product_id . "'");

            foreach ($data['product_related'] as $related_id) {
                $this->db->query("DELETE FROM " . DB_PREFIX . "product_related WHERE product_id = '" . (int)$product_id . "' AND related_id = '" . (int)$related_id . "'");
                $this->db->query("INSERT INTO " . DB_PREFIX . "product_related SET product_id = '" . (int)$product_id . "', related_id = '" . (int)$related_id . "'");
                $this->db->query("DELETE FROM " . DB_PREFIX . "product_related WHERE product_id = '" . (int)$related_id . "' AND related_id = '" . (int)$product_id . "'");
                $this->db->query("INSERT INTO " . DB_PREFIX . "product_related SET product_id = '" . (int)$related_id . "', related_id = '" . (int)$product_id . "'");
            }
        }

        if (isset($data['product_reward'])) {
            $this->db->query("DELETE FROM " . DB_PREFIX . "product_reward WHERE product_id = '" . (int)$product_id . "'");

            foreach ($data['product_reward'] as $customer_group_id => $value) {
                if ((int)$value['points'] > 0) {
                    $this->db->query(
                        "INSERT INTO " . DB_PREFIX . "product_reward SET product_id = '" . (int)$product_id . "',
                         customer_group_id = '" . (int)$customer_group_id . "', points = '" . (int)$value['points'] . "'"
                    );
                }
            }

        }

        // SEO URL

        if (isset($data['product_seo_url'])) {
            $this->db->query("DELETE FROM " . DB_PREFIX . "seo_url WHERE query = 'product_id=" . (int)$product_id . "'");
            foreach ($data['product_seo_url'] as $store_id => $language) {

                foreach ($language as $language_id => $keyword) {
                    if (!empty($keyword)) {
                        $this->db->query(
                            "INSERT INTO " . DB_PREFIX . "seo_url SET store_id = '" . (int)$store_id . "', language_id = '" . (int)$language_id . "',
                             query = 'product_id=" . (int)$product_id . "', keyword = '" . $this->db->escape($keyword) . "'"
                        );
                    }
                }

            }
        }


        if (isset($data['product_layout'])) {
            $this->db->query("DELETE FROM " . DB_PREFIX . "product_to_layout WHERE product_id = '" . (int)$product_id . "'");

            foreach ($data['product_layout'] as $store_id => $layout_id) {
                $this->db->query(
                    "INSERT INTO " . DB_PREFIX . "product_to_layout SET product_id = '" . (int)$product_id . "', store_id = '" . (int)$store_id . "', layout_id = '" . (int)$layout_id . "'"
                );
            }
        }

        $this->cache->delete('product');
    }

    function makeProductDescription($id, $language_id, $description)
    {
        $row = $this->getProductOldDescription($id, $language_id);
        if ($row != false) {
            $row['description'] = $description['description'];
        } else {
            return ['name' => $description['name'], 'description' => $description['description'], 'meta_title' => $meta_title, 'tag' => '', 'meta_keyword' => '', 'meta_description' => ''];
        }
        return $row;
    }

    function getImageWithCurl($link, $local_path)
    {
        $ch = curl_init($link);
        curl_setopt($ch, CURLOPT_HEADER, false);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_BINARYTRANSFER, true);
        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
        $rawdata = curl_exec($ch);
        $http_code = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        curl_close($ch);
        if ($http_code == 200) {
            $fp = fopen($local_path, 'w');
            if (!$fp) {
                return false;
            }
            if (!fwrite($fp, $rawdata)) {
                return false;
            }
            fclose($fp);
            return true;
        }
        return false;

    }

    public function downloadImage($link)
    {
        $link_parts = explode('/', $link);
        $link_parts_length = sizeof($link_parts);

        $sub_path = $link_parts[$link_parts_length - 3] . '/' . $link_parts[$link_parts_length - 2] . '/';

        $path = $this->images_dir . $sub_path;

        if (!is_dir($path)) {
            @mkdir($path, 0777, true);
        }

        $localImage = $path . $link_parts[$link_parts_length - 1];
        $sub_path .= $link_parts[$link_parts_length - 1];

        if (!is_file($localImage) || filesize($localImage) == 0) {
            if ($this->getImageWithCurl($link, $localImage)) {
                return $sub_path;
            } else {
                return false;
            }
        }

        return $sub_path;
    }

    function updateScheduleDate()
    {
        $result['res'] = 0;
        $price_id = isset($_REQUEST['dealer_id']) ? (int)$_REQUEST['dealer_id'] : false;
        $update_hour = isset($_REQUEST['update_hour']) ? (int)$_REQUEST['update_hour'] : -1;
        if ($price_id && $update_hour < 25 && $update_hour > -2) {
            $this->db->query("update " . DB_PREFIX . "4dealer_prices set schedule_update_time=$update_hour where price_id=$price_id");
            $result['res'] = 1;
        }

        return $result;
    }

    function updateScheduleType()
    {
        $price_id = isset($_REQUEST['dealer_id']) ? (int) $_REQUEST['dealer_id'] : false;
        $type = isset($_REQUEST['type']) ? $_REQUEST['type'] : 'full';

        $price = $this->getPriceInfoById($price_id);

        $price['meta']['auto_update_type'] = $type;

        $this->db->query("update " . DB_PREFIX . "4dealer_prices set meta='" . json_encode($price['meta']) . "' where price_id=$price_id");
        $result['res'] = 1;

        return $result;
    }

    public function updatePriceList($price_id, $name, $last_update, $subscription_end)
    {

        return $this->db->query("insert into " . DB_PREFIX . "4dealer_prices(price_id,price_name,4dealer_last_update,subscription_end) 
                                 values($price_id,'$name','$last_update','$subscription_end') 
                                 ON DUPLICATE KEY UPDATE price_name='$name', 4dealer_last_update='$last_update', subscription_end='$subscription_end'");


    }

    public function getLocalPriceList()
    {
        $get_prices = $this->db->query("SELECT * FROM " . DB_PREFIX . "4dealer_prices");
        if ($get_prices->num_rows == 0) {
            return false;
        }

        foreach ($get_prices->rows as &$price) {
            $price['meta'] = isset($price['meta']) ? json_decode($price['meta'], true) : '{}';
            $price['meta'] = array_merge($this->defaultPriceMeta, is_array($price['meta']) ? $price['meta'] : []);
        }

        return $get_prices;
    }

    public function getRemotePriceList()
    {
        $get_prices = $this->getPricesList();
        if (isset($get_prices['Result']) && empty ($get_prices['Error'])) {
            foreach ($get_prices['Result'] as $price) {
                $this->updatePriceList((int)$price['id'], $price['name'], $price['last_update'], $price['till']);
            }
            return $this->getLocalPriceList();
        }
        return false;
    }

    public function getLocalDealerCategories($dealer_id)
    {
        $categories = $this->db->query("SELECT categories FROM " . DB_PREFIX . "4dealer_prices WHERE price_id = {$dealer_id}");

        if (!$categories->row || is_null($categories->row['categories'])) {
            return $this->getRemoteCategories($dealer_id);
        }

        $categories = unserialize($categories->row['categories']);
        $this->sortArrayDealerCategoriesTree($categories);

        return $categories;

    }

    public function getRemoteCategories($dealer_id)
    {
        $categories = $this->getPriceStructure($dealer_id);

        if (sizeof($categories)) {

            $this->sortArrayDealerCategoriesTree($categories);

            $this->db->query(
                "UPDATE " . DB_PREFIX . "4dealer_prices SET categories='" . $this->db->escape(serialize($categories)) . "' 
                 WHERE price_id = $dealer_id"
            );

            return $categories;
        }

        return false;
    }

    function getPriceInfoById($id, $cache = false)
    {
        $cacheKey = 'getPriceInfoById:' . $id;
        $defaultMeta = $this->defaultPriceMeta;

        if (isset($this->cacheSpace[$cacheKey]) && $cache) {
            return $this->cacheSpace[$cacheKey];
        }

        $price = $this->db->query("SELECT * FROM " . DB_PREFIX . "4dealer_prices WHERE price_id = " . (int)$id);

        $result = $this->cacheSpace[$cacheKey] = !$price->num_rows ? false : $price->row;

        if ($result && $result['meta']) {
            $result['meta'] = array_merge($defaultMeta, json_decode($result['meta'], true));
        } else {
            $result['meta'] = $defaultMeta;
        }

        return $this->cacheSpace[$cacheKey] = $result;
    }

    public function getPriceMarkupsByDealerId($id)
    {
        $get_markups = $this->db->query("select * from " . DB_PREFIX . "4dealer_price_markup where price_id=" . (int)$id);
        return !$get_markups->num_rows ? false : $get_markups->rows;

    }

    public function getCategorizedPriceMarkups($id)
    {
        $markups = array();
        $get_markups = $this->getPriceMarkupsByDealerId((int)$id);
        if ($get_markups) {
            foreach ($get_markups as $markup) {
                $markups[$markup['category_id']][] = $markup;
            }
            return $markups;
        }
        return $markups;
    }

    function getCategoriesMappingByPriceId($price_id)
    {
        $result = array();
        $get_mappings = $this->db->query("select term_id,4dealer_cat_id  from " . DB_PREFIX . "4dealer_categories_term where 4dealer_price_id=" . (int)$price_id . " and is_active=1");
        foreach ($get_mappings->rows as $mapping) {
            $result[$mapping['4dealer_cat_id']] = $mapping['term_id'];
        }
        return $result;
    }

    function getDuplicatesData()
    {
        $result = array();
        $get_duid_data = $this->db->query("select product_id, dealer_id, DuID, price, quantity, stock_status_id, model from " . DB_PREFIX . "product where DuID!=0");
        foreach ($get_duid_data->rows as $duid_data) {
            $result[$duid_data['DuID']][] = array('product_id' => $duid_data['product_id'], 'price' => $duid_data['price'], 'dealer_id' => $duid_data['dealer_id'], 'quantity' => $duid_data['quantity'], 'stock_status_id' => $duid_data['stock_status_id'], 'model' => $duid_data['model'], 'DuID' => $duid_data['DuID']);
        }
        unset($get_duid_data);
        return $result;
    }

    function buildFlatCategoriesList($categories, &$categories_list)
    {
        if (isset($categories) && is_array($categories)) {
            foreach ($categories as $category) {
                $categories_list[$category['id']] = $category;
                if (isset($category['children'])) {
                    unset($categories_list[$category['id']]['children']);
                    $this->buildFlatCategoriesList($category['children'], $categories_list);
                }
            }
            return $categories_list;
        }

        return null;
    }

    public function buildCategoryTree(&$elements, $parentId = 0, $parents_info = false)
    {
        $branch = array();
        foreach ($elements as $key => $element) {
            if (!isset($element['parent_id']) || $element['parent_id'] != $parentId || !isset($element['category_id'])) {
                continue;
            }

            if ($element['parent_id'] != $parentId) {
                continue;
            }

            $children = $this->buildCategoryTree($elements, $element['category_id']);
            if (!empty($children)) {
                $element['children'] = $children;
            }
            unset($children);
            if ($parents_info) {
                $element['parents_array'][] = $parentId;
            }
            $branch[$element['category_id']] = $element;
        }
        unset($key, $element);
        return $branch;
    }

    public function searchCategoryById($categories, $id)
    {
        foreach ($categories as $category) {
            if ($category['category_id'] == $id) {
                return $category;
            }
        }
        return false;
    }

    public function dealerCategoriesTreeToTerm($tree, $price_id, $language_id, $parent = 0, $stores = null)
    {
        $this->load->model('catalog/category');


        foreach ($tree as $branch) {
            $name = trim(preg_replace('!\s+!', ' ', $branch['name']));
            $category_description = array($language_id => array('name' => $name, 'description' => '', 'meta_title' => '', 'meta_description' => '', 'meta_keyword' => '',));
            $check_if_cat_exists = $this->db->query("SELECT c.category_id as category_id 
                                          FROM `" . DB_PREFIX . "category_description` d left join `" . DB_PREFIX . "category` c on c.category_id=d.category_id 
                                          where d.`name`='{$name}' and c.`parent_id`='{$parent}' and `language_id`=$language_id");

            if (!$check_if_cat_exists->num_rows) {
                $new_cat_id = $this->model_catalog_category->addCategory(array('parent_id' => $parent, 'column' => 0, 'sort_order' => 0, 'status' => 0, 'category_description' => $category_description, 'status' => 1, 'category_store' => $stores, 'top' => $parent ? 0 : 1));
            } else {
                $new_cat_id = $check_if_cat_exists->row['category_id'];
            }
            if ($new_cat_id) {
                $this->mergeCategories($branch['id'], $new_cat_id, $price_id, false);
            }
            if (isset($branch['children'])) {
                if ($new_cat_id) {
                    $this->dealerCategoriesTreeToTerm($branch['children'], $price_id, $language_id, $new_cat_id, $stores);
                }
            }
        }

        return true;
    }

    function updateDbCategoryMapping($source_id, $target_id, $price_id)
    {
        $this->db->query("DELETE from " . DB_PREFIX . "4dealer_categories_term where 4dealer_price_id=$price_id and 4dealer_cat_id=$source_id");
        $this->db->query("INSERT INTO " . DB_PREFIX . "4dealer_categories_term(term_id,4dealer_price_id,4dealer_cat_id) values($target_id,$price_id,$source_id)");
    }

    function mergeCategories($source_id = false, $target_id = false, $price_id = false, $ajax = true)
    {
        $result = array('res' => 0);
        $source_id = isset($_REQUEST['source_id']) ? (int)$_REQUEST['source_id'] : $source_id;
        $target_id = isset($_REQUEST['target_id']) ? (int)$_REQUEST['target_id'] : $target_id;
        $price_id = isset($_REQUEST['price_id']) ? (int)$_REQUEST['price_id'] : $price_id;

        if (isset($_REQUEST['price_id'])) {
            $categories_list = array();
            $this->buildFlatCategoriesList($this->getRemoteCategories($price_id), $categories_list);
        }
        if ($source_id && $target_id && $price_id) {
            $this->updateDbCategoryMapping($source_id, $target_id, $price_id);
            if (isset($categories_list)) {
                foreach ($categories_list as $category) {
                    if (in_array($source_id, $category['parents'])) {
                        $this->updateDbCategoryMapping($category['id'], $target_id, $price_id);
                    }
                    unset($category);
                }
            }

        }

        if ($ajax) {
            $result['res'] = 1;
            return $result;
        }
        return true;
    }

    /**
     * Получение списка локальных сведенных свойств
     *
     * @param $priceId
     * @return array|false|null
     */
    function getCategoriesTermsList($priceId)
    {
        $sql = "SELECT term_id, `4dealer_cat_id`, `4dealer_price_id`, name FROM " . DB_PREFIX . "4dealer_categories_term 
                JOIN " . DB_PREFIX . "category_description ON term_id = category_id WHERE `4dealer_price_id` = '$priceId' GROUP BY term_id";

        $res = (array) $this->db->query($sql);

        $termIds = array_column($res['rows'], 'term_id');
        $names = array_column($res['rows'], 'name');

        return (array) array_combine($termIds, $names);
    }

    /**
     * Получение 4dealer_cat_id из списка сведенных и переданных $categoryIds
     *
     * @param $priceId
     * @param array $categoryIds
     * @return array
     */
    function getDealerCategoriesByLocalCatIds($priceId, $categoryIds = [])
    {
        $categoryIds = implode(',', $categoryIds);

        $sql = "SELECT term_id, `4dealer_cat_id`, `4dealer_price_id`, name FROM " . DB_PREFIX . "4dealer_categories_term 
                JOIN " . DB_PREFIX . "category_description ON term_id = category_id 
                WHERE `4dealer_price_id` = '$priceId' AND term_id IN ($categoryIds)";

        $res = (array) $this->db->query($sql);

        return (array) array_column($res['rows'], '4dealer_cat_id');
    }

    /**
     * Получение локальной категории из списка сведенных по dealer_cat_id
     *
     * @param int $priceId
     * @param array $dealerCats
     * @return array
     */
    function getLocalCatIdsByDealerCat($priceId, $dealerCats = [])
    {
        $dealerCats = implode(',', $dealerCats);

        $sql = "SELECT term_id FROM " . DB_PREFIX . "4dealer_categories_term 
                WHERE `4dealer_cat_id` IN ($dealerCats) AND `4dealer_price_id` = '$priceId'";

        $res = (array) $this->db->query($sql);

        return (array) array_column($res['rows'], 'term_id');
    }

    function clearMergedCategories()
    {
        $result = array('res' => 0);

        $source_id = isset($_REQUEST['source_id']) ? (int)$_REQUEST['source_id'] : false;
        $price_id = isset($_REQUEST['price_id']) ? (int)$_REQUEST['price_id'] : false;

        if ($source_id && $price_id) {
            if ($this->db->query("DELETE from " . DB_PREFIX . "4dealer_categories_term where 4dealer_price_id=$price_id and 4dealer_cat_id=$source_id")) {
                $result['res'] = 1;
            }
        }

        return $result;
    }

    function setCommandResult($cmd_id, $records, $price_id, $date_filter = false, $cmd = '')
    {
        $timestamp = $this->getCurrentMysqlTime();
        $key = $cmd == 'short_price_update' ? 'local_last_update' : 'local_last_full_update';
        $this->updateJobById($cmd_id, array('status' => 2, 'records' => $records, 'end_time' => $timestamp));

        $this->db->query("update " . DB_PREFIX . "4dealer_prices set $key='$timestamp' where price_id=$price_id");
    }

    function sortBySubkey(&$array, $subkey, $sortType = SORT_ASC)
    {
        $all_keys = array_keys($array);

        array_multisort(array_column($array, $subkey), $sortType, SORT_NUMERIC, $array, $all_keys);
        $array = array_combine($all_keys, $array);
    }

    /**
     * Сортировка по алфавиту (с поддержкой кириллицы)
     *
     * @param $a
     * @param $b
     * @return int
     */
    function mb_strcmp($a, $b) {
        $pattern = '/[А-Яа-яЁё]/u'; // кириллица

        if (!preg_match($pattern, $a) && !preg_match($pattern, $a)) {
            return strcmp($a, $b);
        }

        $la = mb_substr($a, 0, 1, 'utf-8');
        $lb = mb_substr($b, 0, 1, 'utf-8');

        if (ord($la) > 122 && ord($lb) > 122) {
            return $a > $b ? 1 : -1;
        }

        if (ord($la) > 122 || ord($lb) > 122) {
            return $a < $b ? 1 : -1;
        }
    }

    function sortArrayDealerCategoriesTree(&$dealer_categories)
    {
        usort($dealer_categories, function($a, $b) {
            return $this->mb_strcmp($a['name'], $b['name']);
        });

        foreach ($dealer_categories as $key => $dealerCategory) {
            if (isset($dealerCategory['children']) && is_array($dealerCategory['children'])) {
                $this->sortArrayDealerCategoriesTree($dealerCategory['children']);
            }

            $dealer_categories[$key] = $dealerCategory;
        }
    }

    function sortArrayLocalCategoriesTree(&$local_categories)
    {
        usort($local_categories, function($a, $b) {
            if (isset ($a['name']['ru-ru'])) {
                return $this->mb_strcmp($a['name']['ru-ru'], $b['name']['ru-ru']);
            } else {
                return $this->mb_strcmp($a['name']['en-gb'], $b['name']['en-gb']);
            }
        });
    }

    function logger($msg, $logfile = 'oc_4dealer')
    {
        if (!$this->loggerEnabled) {
            return;
        }

        $fileName = DIR_SYSTEM . '/storage/logs/' . date('Y-m-d') . '_' . $logfile . '.log';
        $fp = fopen($fileName, 'a+');

        if (is_array($msg)) {
            fwrite($fp, date('Y-m-d H:i:s - ') . var_export($msg, true) . "\n");
        } else {
            fwrite($fp, date('Y-m-d H:i:s - ') . $msg . "\n");
        }

        fclose($fp);
    }
    
    function deleteOldLogFiles ($livetime, $logfile = 'oc_4dealer') 
    {
        if ( intval($livetime) > 0 )
        {
            $dirName = DIR_SYSTEM . '/storage/logs/';
            $oldDate = date('Y-m-d',strtotime("-{$livetime} day"));
            $pattern = "/".$oldDate."_".$logfile.".log/";
            if ($handle = opendir($dirName)) {
                while (false !== ($entry = readdir($handle))) {
                    if ($entry != "." && $entry != ".." && preg_match($pattern, $entry)) {
                        unlink($_SERVER['DOCUMENT_ROOT'].$dir.$entry);
                    }
                }
                closedir($handle);
            }
        }
    }
}

/**
 * This file is part of the array_column library
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 *
 * @copyright Copyright (c) Ben Ramsey (http://benramsey.com)
 * @license http://opensource.org/licenses/MIT MIT
 */
if (!function_exists('array_column')) {
    /**
     * Returns the values from a single column of the input array, identified by
     * the $columnKey.
     *
     * Optionally, you may provide an $indexKey to index the values in the returned
     * array by the values from the $indexKey column in the input array.
     *
     * @param array $input A multi-dimensional array (record set) from which to pull
     *                     a column of values.
     * @param mixed $columnKey The column of values to return. This value may be the
     *                         integer key of the column you wish to retrieve, or it
     *                         may be the string key name for an associative array.
     * @param mixed $indexKey (Optional.) The column to use as the index/keys for
     *                        the returned array. This value may be the integer key
     *                        of the column, or it may be the string key name.
     * @return array
     */
    function array_column($input = null, $columnKey = null, $indexKey = null)
    {
        // Using func_get_args() in order to check for proper number of
        // parameters and trigger errors exactly as the built-in array_column()
        // does in PHP 5.5.
        $argc = func_num_args();
        $params = func_get_args();
        if ($argc < 2) {
            trigger_error("array_column() expects at least 2 parameters, {$argc} given", E_USER_WARNING);
            return null;
        }
        if (!is_array($params[0])) {
            trigger_error(
                'array_column() expects parameter 1 to be array, ' . gettype($params[0]) . ' given',
                E_USER_WARNING
            );
            return null;
        }
        if (!is_int($params[1])
            && !is_float($params[1])
            && !is_string($params[1])
            && $params[1] !== null
            && !(is_object($params[1]) && method_exists($params[1], '__toString'))
        ) {
            trigger_error('array_column(): The column key should be either a string or an integer', E_USER_WARNING);
            return false;
        }
        if (isset($params[2])
            && !is_int($params[2])
            && !is_float($params[2])
            && !is_string($params[2])
            && !(is_object($params[2]) && method_exists($params[2], '__toString'))
        ) {
            trigger_error('array_column(): The index key should be either a string or an integer', E_USER_WARNING);
            return false;
        }
        $paramsInput = $params[0];
        $paramsColumnKey = ($params[1] !== null) ? (string) $params[1] : null;
        $paramsIndexKey = null;
        if (isset($params[2])) {
            if (is_float($params[2]) || is_int($params[2])) {
                $paramsIndexKey = (int) $params[2];
            } else {
                $paramsIndexKey = (string) $params[2];
            }
        }
        $resultArray = array();
        foreach ($paramsInput as $row) {
            $key = $value = null;
            $keySet = $valueSet = false;
            if ($paramsIndexKey !== null && array_key_exists($paramsIndexKey, $row)) {
                $keySet = true;
                $key = (string) $row[$paramsIndexKey];
            }
            if ($paramsColumnKey === null) {
                $valueSet = true;
                $value = $row;
            } elseif (is_array($row) && array_key_exists($paramsColumnKey, $row)) {
                $valueSet = true;
                $value = $row[$paramsColumnKey];
            }
            if ($valueSet) {
                if ($keySet) {
                    $resultArray[$key] = $value;
                } else {
                    $resultArray[] = $value;
                }
            }
        }
        return $resultArray;
    }
}

if (version_compare(VERSION, '3.0', '>=')) {
    class ModelExtensionForDealerImport extends ModelToolForDealerImport
    {
    }
}

